@@ -24,5 +24,10 @@ mbdesktop_view_paint_list(MBDesktop *mb, MBPixbufImage *dest_img);
 void 
 mbdesktop_view_paint_icons(MBDesktop *mb, MBPixbufImage *img_dest);
 
+void
+mbdesktop_view_item_highlight (MBDesktop     *mb, 
+			       MBDesktopItem *item,
+			       int            highlight_style);
+
 
 #endif
