@@ -4,11 +4,17 @@
 #include "dock.h"
 
 #ifdef DEBUG
-#define PANELFILE "/.mbdock.debug"
+#define PANELFILE "mbdock.session.debug"
 #else
-#define PANELFILE "/.mbdock"
+#define PANELFILE "mbdock.session"
 #endif
 
+void
+session_destroy(MBPanel *panel);
+
+void 
+session_set_defaults(MBPanel *panel, char *defaults);
+
 void session_init(MBPanel *panel);
 
 void session_save(MBPanel *panel);
