@@ -17,18 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef __MB__H__
-#define __MB__H__
 
-#ifndef MB_API_NOT_YET_FROZEN
-#error "LibMB should only be used if you understand that it's subject to frequent change, and is not yet supported as a fixed API/ABI"
-#endif
-
-
-#include "<mb/mbtray.h>"
-#include "<mb/mbpixbuf.h>"
-#include "<mb/mbutil.h>"
-#include "<mb/mbmenu.h>"
-#include "<mb/mbdotdesktop.h>"
-
-#endif
+#include "mbtray.h"
+#include "mbpixbuf.h"
+#include "mbutil.h"
+#include "mbmenu.h"
+#include "mbdotdesktop.h"
