@@ -63,16 +63,6 @@ desktop_client_configure(Client *c)
 void
 desktop_client_show(Client *c)
 {
-   XLowerWindow(c->wm->dpy, c->window);
-   XMapWindow(c->wm->dpy, c->window);
-
-   if (client_want_focus(c))
-   {
-     dbg("Desktop setting focus\n");
-      XSetInputFocus(c->wm->dpy, c->window,
-		     RevertToPointerRoot, CurrentTime);
-      c->wm->focused_client = c;
-   }
-
+  wm_toggle_desktop(c->wm);
 }
 
