@@ -45,6 +45,29 @@ dialog_client_new(Wm *w, Window win, Client *trans)
    return c;
 }
 
+static void
+dialog_client_get_offsets(Client *c, int *e, int *s, int *w)
+{
+#ifdef USE_MSG_WIN
+  if (c->flags & CLIENT_IS_MESSAGE_DIALOG)
+    {
+      *s = theme_frame_defined_height_get(c->wm->mbtheme, 
+				       FRAME_MSG_SOUTH);
+      *e = theme_frame_defined_width_get(c->wm->mbtheme, 
+					 FRAME_MSG_EAST );
+      *w = theme_frame_defined_width_get(c->wm->mbtheme, 
+					 FRAME_MSG_WEST );
+      return;
+    }
+#endif 
+   *s = theme_frame_defined_height_get(c->wm->mbtheme, 
+				       FRAME_DIALOG_SOUTH);
+   *e = theme_frame_defined_width_get(c->wm->mbtheme, 
+				      FRAME_DIALOG_EAST );
+   *w = theme_frame_defined_width_get(c->wm->mbtheme, 
+				      FRAME_DIALOG_WEST );
+}
+
 static void
 dialog_client_check_for_state_hints(Client *c)
 {
@@ -70,12 +93,9 @@ void
 dialog_client_move_resize(Client *c)
 {
    int frm_size     = dialog_client_title_height(c);
-   int offset_south = theme_frame_defined_height_get(c->wm->mbtheme, 
-						     FRAME_DIALOG_SOUTH);
-   int offset_east  = theme_frame_defined_width_get(c->wm->mbtheme, 
-						    FRAME_DIALOG_EAST );
-   int offset_west  = theme_frame_defined_width_get(c->wm->mbtheme, 
-						    FRAME_DIALOG_WEST );
+   int offset_south = 0, offset_west = 0, offset_east = 0;
+
+   dialog_client_get_offsets(c, &offset_east, &offset_south, &offset_west);
 
    base_client_move_resize(c);
 
@@ -126,6 +146,18 @@ dialog_client_title_height(Client *c)
 {
    if (c->flags & CLIENT_TITLE_HIDDEN_FLAG)
       return 0;
+
+#ifdef USE_MSG_WIN
+  if (c->flags & CLIENT_IS_MESSAGE_DIALOG)
+    {
+      return theme_frame_defined_height_get(c->wm->mbtheme, FRAME_MSG);
+    }
+#endif
+
+  if (c->flags & CLIENT_BORDERS_ONLY_FLAG
+      && theme_has_frame_type_defined(c->wm->mbtheme, FRAME_DIALOG_NORTH))    
+    return theme_frame_defined_height_get(c->wm->mbtheme, FRAME_DIALOG_NORTH);
+
    return theme_frame_defined_height_get(c->wm->mbtheme, FRAME_DIALOG);
 }
 
@@ -139,72 +171,93 @@ dialog_client_show(Client *c)
   XMapSubwindows(c->wm->dpy, c->frame);
   XMapRaised(c->wm->dpy, c->frame);
   
-  if (client_want_focus(c))
+  if (client_want_focus(c) && (!(c->flags & CLIENT_IS_MESSAGE_DIALOG)))
     {
       XSetInputFocus(c->wm->dpy, c->window,
 		     RevertToPointerRoot, CurrentTime);
       c->wm->focused_client = c;
     }
+
+#ifdef USE_MSG_WIN 		/* Ewe! mesg window hacks :/  */
+  if (c->wm->config->dialog_shade 
+      && (c->flags & CLIENT_IS_MODAL_FLAG)
+      && c->wm->msg_win_queue_head)
+    {
+        Client *msg_client = NULL;
+	if ((msg_client = wm_find_client(c->wm, 
+					 c->wm->msg_win_queue_head->win, 
+					 WINDOW)) != NULL)
+	  msg_client->show(msg_client);
+    }
+#endif
+
   c->mapped = True;
 }
 
+
 void
 dialog_client_reparent(Client *c)
 {
    XSetWindowAttributes attr;
 
    int offset_north = dialog_client_title_height(c);
-   int offset_south = theme_frame_defined_height_get(c->wm->mbtheme, 
-						     FRAME_DIALOG_SOUTH);
-   int offset_east  = theme_frame_defined_width_get(c->wm->mbtheme, 
-						    FRAME_DIALOG_EAST );
-   int offset_west  = theme_frame_defined_width_get(c->wm->mbtheme, 
-						    FRAME_DIALOG_WEST );
+   int offset_south = 0, offset_west = 0, offset_east = 0;
 
-   if (c->flags & CLIENT_TITLE_HIDDEN_FLAG)
-   {
-      c->frame = c->title_frame = c->window;
-      return;
-   }
+   dialog_client_get_offsets(c, &offset_east, &offset_south, &offset_west);
 
    attr.override_redirect = True;
    attr.background_pixel  = BlackPixel(c->wm->dpy, c->wm->screen);
    attr.event_mask        = ChildMask|ButtonPressMask|ExposureMask;
 
+   dbg("%s() want lowlight : wm:%i , client:%i\n", __func__,
+       c->wm->config->dialog_shade, (c->flags & CLIENT_IS_MODAL_FLAG));
+
    if (c->wm->config->dialog_shade && (c->flags & CLIENT_IS_MODAL_FLAG))
      {
-        wm_lowlight(c->wm, c);
+       dbg("%s() LOWLIGHTING\n", __func__);
+       wm_lowlight(c->wm, c);
      }
    else
      {
-       c->frame =
+       if (c->flags & CLIENT_TITLE_HIDDEN_FLAG) 
+	 {
+	   c->frame = c->window;
+	 }
+       else c->frame = XCreateWindow(c->wm->dpy, 
+				     c->wm->root, 
+				     0, 0,
+				     c->width + offset_east + offset_west, 
+				     c->height + offset_north + offset_south, 
+				     0,
+				     CopyFromParent, 
+				     CopyFromParent, 
+				     CopyFromParent,
+				     CWOverrideRedirect|CWEventMask|CWBackPixel,
+				     &attr);
+     }
+
+
+   attr.background_pixel = BlackPixel(c->wm->dpy, c->wm->screen);
+
+   if (c->flags & CLIENT_TITLE_HIDDEN_FLAG)
+     {
+       c->title_frame = c->window;
+     }
+   else
+     {
+       c->title_frame =
 	 XCreateWindow(c->wm->dpy, 
-		       c->wm->root, 
-		       0, 0,
+		       c->frame, 
+		       0, 0, 
 		       c->width + offset_east + offset_west, 
 		       c->height + offset_north + offset_south, 
 		       0,
 		       CopyFromParent, 
 		       CopyFromParent, 
 		       CopyFromParent,
-		       CWOverrideRedirect|CWEventMask|CWBackPixel,
+		       CWOverrideRedirect|CWBackPixel|CWEventMask, 
 		       &attr);
      }
-
-   attr.background_pixel = BlackPixel(c->wm->dpy, c->wm->screen);
-   
-   c->title_frame =
-     XCreateWindow(c->wm->dpy, 
-		   c->frame, 
-		   0, 0, 
-		   c->width + offset_east + offset_west, 
-		   c->height + offset_north + offset_south, 
-		   0,
-		   CopyFromParent, 
-		   CopyFromParent, 
-		   CopyFromParent,
-		   CWOverrideRedirect|CWBackPixel|CWEventMask, 
-		   &attr);
    
    XSetWindowBorderWidth(c->wm->dpy, c->window, 0);
 
@@ -212,13 +265,10 @@ dialog_client_reparent(Client *c)
 
    XSelectInput(c->wm->dpy, c->window,
 		ButtonPressMask|ColormapChangeMask|PropertyChangeMask);
-   
-   XReparentWindow(c->wm->dpy, 
-		   c->window, 
-		   c->frame,
-		   offset_west, 
-		   offset_north);
 
+   if (c->frame != c->window)
+     XReparentWindow(c->wm->dpy, c->window, c->frame, 
+		     offset_west, offset_north);
 }
 
 
@@ -227,20 +277,23 @@ dialog_client_configure(Client *c)
 {
   int side_frame_width = 0, bottom_frame_width = 0;
   int max_w = 0, max_h = 0, toolbar_y = 0; 
+  int offset_south = 0, offset_west = 0, offset_east = 0;
 
+  dialog_client_get_offsets(c, &offset_east, &offset_south, &offset_west);
 
   if (!(c->flags & CLIENT_TITLE_HIDDEN_FLAG))
     {
-      side_frame_width = 
-	theme_frame_defined_width_get(c->wm->mbtheme, FRAME_DIALOG_EAST)
-	+ theme_frame_defined_width_get(c->wm->mbtheme, FRAME_DIALOG_WEST);
-
-      bottom_frame_width =
-	theme_frame_defined_height_get(c->wm->mbtheme, FRAME_DIALOG_SOUTH);
+      side_frame_width = offset_east + offset_west;
+      bottom_frame_width = offset_south;
     }
 
   toolbar_y = c->wm->dpy_height - wm_get_offsets_size(c->wm, SOUTH, NULL,True);
 
+  /* Toolbars dont effect messages and splash windows */
+  if ((c->flags & CLIENT_TITLE_HIDDEN_FLAG) 
+      || (c->flags & CLIENT_IS_MESSAGE_DIALOG))
+    toolbar_y = c->wm->dpy_height; 
+
   max_w = c->wm->dpy_width - side_frame_width - 16;
 
    /* make sure the dialog doesn't cover any toolbars */
@@ -248,7 +301,21 @@ dialog_client_configure(Client *c)
 
   dbg("dialog wants %i x %i, %i x %i", c->x, c->y, c->width, c->height);
   
-  if (c->width > max_w) c->width   = max_w;
+  /* 
+     if statergy = no intervention , return; 
+
+     if startergy = fullwidth && not splash or decorationless
+        set x = 0 - border_east , w = display width  
+	
+	XXXX move above toolbars 
+	return 
+
+     else
+  */
+
+  if (c->wm->config->dialog_stratergy == WM_DIALOGS_STRATERGY_FREE
+      || (c->flags & CLIENT_IS_MESSAGE_DIALOG) )
+    return;
 
    if (c->height > max_h) 
      {
@@ -262,6 +329,17 @@ dialog_client_configure(Client *c)
      {
        c->y -= (( c->y + c->height + bottom_frame_width ) - toolbar_y );
      }
+
+  if (c->wm->config->dialog_stratergy == WM_DIALOGS_STRATERGY_CONSTRAINED_HORIZ
+      && !(c->flags & CLIENT_TITLE_HIDDEN_FLAG) )
+    {
+      c->x = 0 - offset_east;
+      c->width = c->wm->dpy_width;
+
+      return;
+    }
+
+  if (c->width > max_w) c->width   = max_w;
    
    /* only repos if at 0,0 or off screen  */
    if ( !c->x || (c->x + c->width) > c->wm->dpy_width || (c->x < 0))
@@ -277,15 +355,33 @@ dialog_client_redraw(Client *c, Bool use_cache)
   Bool is_shaped = False;
 
   int offset_north = dialog_client_title_height(c);
-  int offset_south = theme_frame_defined_height_get(c->wm->mbtheme, 
-						    FRAME_DIALOG_SOUTH);
-  int offset_east  = theme_frame_defined_width_get(c->wm->mbtheme, 
-						   FRAME_DIALOG_EAST );
-  int offset_west  = theme_frame_defined_width_get(c->wm->mbtheme, 
-						   FRAME_DIALOG_WEST );
+  int offset_south = 0, offset_west = 0, offset_east = 0;
+  int total_w = 0, total_h = 0;
+
+  int frame_ref_top   = FRAME_DIALOG;
+  int frame_ref_east  = FRAME_DIALOG_EAST;
+  int frame_ref_west  = FRAME_DIALOG_WEST;
+  int frame_ref_south = FRAME_DIALOG_SOUTH;
+
+  dialog_client_get_offsets(c, &offset_east, &offset_south, &offset_west);
+
+  total_w = offset_east  + offset_west + c->width;
+  total_h = offset_north + offset_south + c->height;
+
+
+  if (c->flags & CLIENT_BORDERS_ONLY_FLAG 
+      && theme_has_frame_type_defined(c->wm->mbtheme, FRAME_DIALOG_NORTH))
+    frame_ref_top   = FRAME_DIALOG_NORTH;
+
+  /* 'message dialogs have there own decorations */
+  if (c->flags & CLIENT_IS_MESSAGE_DIALOG)
+    {
+      frame_ref_top   = FRAME_MSG;
+      frame_ref_east  = FRAME_MSG_EAST;
+      frame_ref_west  = FRAME_MSG_WEST;
+      frame_ref_south = FRAME_MSG_SOUTH;
+    }
 
-  int total_w = offset_east  + offset_west + c->width;
-  int total_h = offset_north + offset_south + c->height;
 
   dbg("%s() c->width : %i , offset_east : %i, offset_west : %i\n",
       __func__, c->width, offset_east, offset_west );
@@ -294,7 +390,7 @@ dialog_client_redraw(Client *c, Bool use_cache)
 
    if (use_cache && c->backing != None) return;
 
-  is_shaped = theme_frame_wants_shaped_window( c->wm->mbtheme, FRAME_DIALOG);
+  is_shaped = theme_frame_wants_shaped_window( c->wm->mbtheme, frame_ref_top);
   
   if (c->backing == (Pixmap)NULL)
     client_init_backing(c, total_w, total_h);
@@ -307,21 +403,23 @@ dialog_client_redraw(Client *c, Bool use_cache)
   XFillRectangle(c->wm->dpy, c->backing, c->wm->mbtheme->gc, 
 		 0, 0, total_w, total_h);
 
-  theme_frame_paint(c->wm->mbtheme, c, FRAME_DIALOG, 
+  theme_frame_paint(c->wm->mbtheme, c, frame_ref_top, 
 		    0, 0, total_w, offset_north); 
     
-  theme_frame_paint(c->wm->mbtheme, c, FRAME_DIALOG_WEST, 
+  theme_frame_paint(c->wm->mbtheme, c, frame_ref_west, 
 		    0, offset_north, offset_west, c->height); 
   
-  theme_frame_paint(c->wm->mbtheme, c, FRAME_DIALOG_EAST, 
+  theme_frame_paint(c->wm->mbtheme, c, frame_ref_east, 
 		    total_w - offset_east, offset_north, 
 		    offset_east, c->height); 
 
-  theme_frame_paint(c->wm->mbtheme, c, FRAME_DIALOG_SOUTH, 
+  theme_frame_paint(c->wm->mbtheme, c, frame_ref_south, 
 		    0, total_h - offset_south, 
 		    total_w, offset_south); 
 
-  if (!(c->flags & CLIENT_BORDERS_ONLY_FLAG))
+  /* We dont paint buttons for borderonly and message dialogs */
+  if (!(c->flags & CLIENT_BORDERS_ONLY_FLAG
+	|| c->flags & CLIENT_IS_MESSAGE_DIALOG))
     {
       theme_frame_button_paint(c->wm->mbtheme, c, 
 			       BUTTON_ACTION_CLOSE, 
@@ -375,10 +473,11 @@ void
 dialog_client_button_press(Client *c, XButtonEvent *e)
 {
   int offset_north = dialog_client_title_height(c);
-  int offset_west = theme_frame_defined_width_get(c->wm->mbtheme, 
-						    FRAME_DIALOG_WEST);
-  int offset_east  = theme_frame_defined_width_get(c->wm->mbtheme, 
-						   FRAME_DIALOG_EAST );
+  int offset_south = 0, offset_west = 0, offset_east = 0;
+
+  if (c->flags & CLIENT_IS_MESSAGE_DIALOG) return;
+
+  dialog_client_get_offsets(c, &offset_east, &offset_south, &offset_west);
 
   dbg("%s() c->width : %i , offset_east : %i, offset_west : %i\n",
       __func__, c->width, offset_east, offset_west );
@@ -391,10 +490,10 @@ dialog_client_button_press(Client *c, XButtonEvent *e)
 	 client_deliver_delete(c);
 	 break;
       case BUTTON_ACTION_HELP:
-	client_deleiver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_HELP]);
+	client_deliver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_HELP]);
 	 break;
       case BUTTON_ACTION_ACCEPT:
-	client_deleiver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_ACCEPT]);
+	client_deliver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_ACCEPT]);
       case -1: 		 /* Cancelled  */
 	 break;
       case 0:
@@ -412,14 +511,13 @@ dialog_client_drag(Client *c) /* drag box */
   int old_cy = c->y;
     
   int frm_size     = dialog_client_title_height(c);
-  int offset_south = theme_frame_defined_height_get(c->wm->mbtheme, 
-						    FRAME_DIALOG_SOUTH);
-  int offset_east  = theme_frame_defined_width_get(c->wm->mbtheme, 
-						   FRAME_DIALOG_EAST );
-  int offset_west  = theme_frame_defined_width_get(c->wm->mbtheme, 
-						   FRAME_DIALOG_WEST );
+  int offset_south = 0, offset_west = 0, offset_east = 0;
 
-  XMapRaised(c->wm->dpy, c->frame);
+#ifdef USE_MSG_WIN
+  Client *t = NULL;
+#endif 
+
+  dialog_client_get_offsets(c, &offset_east, &offset_south, &offset_west);
 
   if (XGrabPointer(c->wm->dpy, c->wm->root, False,
 		   (ButtonPressMask|ButtonReleaseMask|PointerMotionMask),
@@ -448,7 +546,8 @@ dialog_client_drag(Client *c) /* drag box */
 		    c->width + offset_west + offset_east,
 		    c->height + frm_size + offset_south);
 
-      c->x = old_cx + (ev.xmotion.x - x1);
+      if (c->wm->config->dialog_stratergy != WM_DIALOGS_STRATERGY_CONSTRAINED_HORIZ)
+	c->x = old_cx + (ev.xmotion.x - x1);
       c->y = old_cy + (ev.xmotion.y - y1);
 
       _draw_outline(c, c->x - offset_west, c->y - frm_size,
@@ -480,6 +579,21 @@ dialog_client_drag(Client *c) /* drag box */
 	  XMoveWindow(c->wm->dpy, c->frame, c->x,
 		      c->y - dialog_client_title_height(c));
 	}
+
+      XMapRaised(c->wm->dpy, c->frame);
+
+#ifdef USE_MSG_WIN
+      /* Check for message super above dialogs */
+      START_CLIENT_LOOP(c->wm, t)
+	{
+	  if (t->flags & CLIENT_IS_MESSAGE_DIALOG)
+	    {
+	      t->show(t);
+	      break;
+	    }
+	}
+      END_CLIENT_LOOP(c->wm, t);
+#endif 
       XUngrabPointer(c->wm->dpy, CurrentTime);
       XUngrabServer(c->wm->dpy);
       return;
@@ -507,11 +621,20 @@ _draw_outline(Client *c, int x, int y, int w, int h)
  
 void dialog_client_destroy(Client *c)
 {
-  /*   if (c->wm->main_client != NULL)
-       c->wm->main_client->show(c->wm->main_client); */
-   base_client_destroy(c);
-   /*
-   if (!(c->wm->flags & DESKTOP_RAISED_FLAG)) 
-     wm_activate_client(c->wm->main_client);
-   */
+#ifdef USE_MSG_WIN
+  Wm *w = c->wm; 
+  int was_msg = (c->flags & CLIENT_IS_MESSAGE_DIALOG);
+#endif 
+
+  base_client_destroy(c);
+
+#ifdef USE_MSG_WIN
+
+   if (was_msg) 
+     {
+       dbg("%s() was message poping queue\n", __func__);
+       wm_msg_win_queue_pop(w);
+     }
+#endif 
+   
 }
