@@ -14,109 +14,110 @@ TODO:
 void
 ewmh_init(Wm *w)
 {
-   int num_desktops = 1;
-   
-   w->atoms[WINDOW_TYPE_TOOLBAR]
-      = XInternAtom(w->dpy, "_NET_WM_WINDOW_TYPE_TOOLBAR",False);
-   w->atoms[WINDOW_TYPE_DOCK]
-      = XInternAtom(w->dpy, "_NET_WM_WINDOW_TYPE_DOCK",False);
-   w->atoms[WINDOW_TYPE_DIALOG]
-      = XInternAtom(w->dpy, "_NET_WM_WINDOW_TYPE_DIALOG",False);
-   w->atoms[WINDOW_TYPE_SPLASH]
-      = XInternAtom(w->dpy, "_NET_WM_WINDOW_TYPE_SPLASH",False);
+  /*  
+      The list below *MUST* be kept in the same order as the corresponding
+      emun in structs.h or *everything* will break.
+      Doing it like this avoids a mass of round trips on startup.
+  */
 
-   w->atoms[WINDOW_TYPE_DESKTOP]
-      = XInternAtom(w->dpy, "_NET_WM_WINDOW_TYPE_DESKTOP",False);
-   w->atoms[WINDOW_STATE]
-      = XInternAtom(w->dpy, "_NET_WM_STATE",False);
-   w->atoms[WINDOW_STATE_FULLSCREEN]
-      = XInternAtom(w->dpy, "_NET_WM_STATE_FULLSCREEN",False);
-   w->atoms[WINDOW_STATE_MODAL]
-      = XInternAtom(w->dpy, "_NET_WM_STATE_MODAL",False);
-   w->atoms[_NET_SUPPORTED]
-      = XInternAtom(w->dpy, "_NET_SUPPORTED",False);
-   w->atoms[_NET_CLIENT_LIST]
-      = XInternAtom(w->dpy, "_NET_CLIENT_LIST",False);
-   w->atoms[_NET_NUMBER_OF_DESKTOPS]
-      = XInternAtom(w->dpy, "_NET_NUMBER_OF_DESKTOPS",False);
-   w->atoms[_NET_ACTIVE_WINDOW]
-      = XInternAtom(w->dpy, "_NET_ACTIVE_WINDOW",False);
-   w->atoms[_NET_SUPPORTING_WM_CHECK]
-      = XInternAtom(w->dpy, "_NET_SUPPORTING_WM_CHECK",False);
-   w->atoms[_WIN_SUPPORTING_WM_CHECK]
-      = XInternAtom(w->dpy, "_WIN_SUPPORTING_WM_CHECK",False);
-   w->atoms[_NET_CLOSE_WINDOW]
-      = XInternAtom(w->dpy, "_NET_CLOSE_WINDOW",False);
-   w->atoms[_NET_WM_NAME]
-      = XInternAtom(w->dpy, "_NET_WM_NAME",False);
-   w->atoms[UTF8_STRING]
-      = XInternAtom(w->dpy, "UTF8_STRING",False);
+  char *atom_names[] = {
+    "WM_STATE",
+    "WM_CHANGE_STATE",
+    "WM_PROTOCOLS",
+    "WM_DELETE_WINDOW",
+    "WM_COLORMAP_WINDOWS",
+    "_MB_THEME",
+    "_MB_THEME_NAME",
 
-   w->atoms[_NET_CLIENT_LIST_STACKING]
-      = XInternAtom(w->dpy, "_NET_CLIENT_LIST_STACKING",False);
-   w->atoms[_NET_CURRENT_DESKTOP]
-      = XInternAtom(w->dpy, "_NET_CURRENT_DESKTOP",False);
-   w->atoms[_NET_WM_DESKTOP]
-      = XInternAtom(w->dpy, "_NET_WM_DESKTOP",False);
+    "_NET_WM_WINDOW_TYPE_TOOLBAR",
+    "_NET_WM_WINDOW_TYPE_DOCK",
+    "_NET_WM_WINDOW_TYPE_DIALOG",
+    "_NET_WM_WINDOW_TYPE_SPLASH",
+    "_NET_WM_WINDOW_TYPE_DESKTOP",
+  
+    "_MB_COMMAND",
 
-   /* set to screen space minus dock */
-   w->atoms[_NET_DESKTOP_GEOMETRY]
-      = XInternAtom(w->dpy, "_NET_DESKTOP_GEOMETRY",False);
-   w->atoms[_NET_WORKAREA]
-      = XInternAtom(w->dpy, "_NET_WORKAREA",False);
+    "_MB_CLIENT_EXEC_MAP",
+    "_MB_CLIENT_STARTUP_LIST",
+  
+    "_NET_WM_STATE",
+    "_NET_WM_STATE_FULLSCREEN",
+    "_NET_WM_STATE_MODAL",
+  
+    "_NET_SUPPORTED",
+    "_NET_CLIENT_LIST",
+    "_NET_NUMBER_OF_DESKTOPS",
+    "_NET_ACTIVE_WINDOW",
+    "_NET_SUPPORTING_WM_CHECK",
+  
+    "_NET_CLOSE_WINDOW",
+  
+    "_NET_WM_NAME",
+    "UTF8_STRING",
+    "_NET_CLIENT_LIST_STACKING",
+    "_NET_CURRENT_DESKTOP",
+    "_NET_WM_DESKTOP",
+    "_NET_WM_ICON",
+    "_NET_DESKTOP_GEOMETRY",
+    "_NET_WORKAREA",
+  
+    "_NET_SHOW_DESKTOP",
+  
+    "_MOTIF_WM_HINTS",
 
-   w->atoms[_NET_CLOSE_WINDOW]
-      = XInternAtom(w->dpy, "_NET_CLOSE_WINDOW",False);
+    "_NET_WM_CONTEXT_HELP",
+    "_NET_WM_CONTEXT_ACCEPT",
 
-   w->atoms[_NET_SHOW_DESKTOP]
-      = XInternAtom(w->dpy, "_NET_SHOWING_DESKTOP",False);
+    "_NET_WM_ALLOWED_ACTIONS",
+    "_NET_WM_ACTION_MOVE",
+    "_NET_WM_ACTION_RESIZE",
+    "_NET_WM_ACTION_MINIMIZE",
+    "_NET_WM_ACTION_SHADE",
+    "_NET_WM_ACTION_STICK",
+    "_NET_WM_ACTION_MAXIMIZE_HORZ",
+    "_NET_WM_ACTION_MAXIMIZE_VERT",
+    "_NET_WM_ACTION_FULLSCREEN",
+    "_NET_WM_ACTION_CHANGE_DESKTOP",
+    "_NET_WM_ACTION_CLOSE",
+    
+    "_NET_STARTUP_ID",
 
-   w->atoms[_NET_WM_ICON]
-     = XInternAtom(w->dpy, "_NET_WM_ICON", False);
-   
-   /* Matchbox dock only hints */
-   w->atoms[MB_COMMAND]
-      = XInternAtom(w->dpy, "_MB_COMMAND", False);
-   w->atoms[MB_CLIENT_EXEC_MAP]
-      = XInternAtom(w->dpy, "_MB_CLIENT_EXEC_MAP", False);
-   w->atoms[MB_CLIENT_STARTUP_LIST]
-      = XInternAtom(w->dpy, "_MB_CLIENT_STARTUP_LIST", False);
+    "_WIN_SUPPORTING_WM_CHECK",
 
+    "_MB_WM_WINDOW_TYPE_MESSAGE",
+    "_MB_WM_WINDOW_TYPE_MESSAGE_TIMEOUT",
 
-   /* Old school Motif */
+    "_NET_WM_WINDOW_TYPE",
 
-   w->atoms[_MOTIF_WM_HINTS]
-      = XInternAtom(w->dpy, "_MOTIF_WM_HINTS", False);
+    "_NET_WM_PING",
+    "_NET_WM_PID",
+    "WM_CLIENT_MACHINE"
+  };
 
-   /* KDE / OPIE style hints for 'special' decorations */
-   w->atoms[_NET_WM_CONTEXT_HELP]
-      = XInternAtom(w->dpy, "_NET_WM_CONTEXT_HELP", False);
+  XInternAtoms (w->dpy, atom_names, ATOM_COUNT,
+                False, w->atoms);
 
-   w->atoms[_NET_WM_CONTEXT_ACCEPT]
-      = XInternAtom(w->dpy, "_NET_WM_CONTEXT_ACCEPT", False);
+}
 
-   /* startup-notification */
-
-   w->atoms[_NET_STARTUP_ID]
-      = XInternAtom(w->dpy, "_NET_STARTUP_ID", False);
-
-
-   set_compliant(w);
-   set_supported(w);
-
-   XChangeProperty(w->dpy, w->root, w->atoms[_NET_NUMBER_OF_DESKTOPS],
-		   XA_CARDINAL, 32, PropModeReplace,
-		   (unsigned char *)&num_desktops, 1);
-
-   XChangeProperty(w->dpy, w->root, w->atoms[_NET_CURRENT_DESKTOP],
-		   XA_CARDINAL, 32, PropModeReplace,
-		   (unsigned char *)&num_desktops, 1);
-
-   
+void
+ewmh_init_props(Wm *w)
+{
+  int num_desktops = 1;
+  
+  set_compliant(w);
+  set_supported(w);
+  
+  XChangeProperty(w->dpy, w->root, w->atoms[_NET_NUMBER_OF_DESKTOPS],
+		  XA_CARDINAL, 32, PropModeReplace,
+		  (unsigned char *)&num_desktops, 1);
+  
+  XChangeProperty(w->dpy, w->root, w->atoms[_NET_CURRENT_DESKTOP],
+		  XA_CARDINAL, 32, PropModeReplace,
+		  (unsigned char *)&num_desktops, 1);
 }
 
 int
-ewhm_handle_root_message(Wm *w, XClientMessageEvent *e)
+ewmh_handle_root_message(Wm *w, XClientMessageEvent *e)
 {
   /* Handle client messages _sent_ to root window */
 
@@ -137,18 +138,28 @@ ewhm_handle_root_message(Wm *w, XClientMessageEvent *e)
 	 if ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
 	    client_deliver_delete(c);
 	 return 1;
-
+   } else if (e->message_type == w->atoms[WM_PROTOCOLS]
+	      && e->data.l[0] == w->atoms[_NET_WM_PING]) {
+     if ((c = wm_find_client(w, e->data.l[1], WINDOW)) != NULL)
+       {
+	 dbg("%s() pong from %s\n", __func__, c->name);
+	 if (c->pings_pending > 0) c->pings_pending--;
+       }
    } else if (e->message_type == w->atoms[WINDOW_STATE]) {
-     if (e->data.l[0] == w->atoms[WINDOW_STATE_FULLSCREEN]
+     if (e->data.l[1] == w->atoms[WINDOW_STATE_FULLSCREEN]
 	 && ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
 	 && c->type == mainwin)
        {
 	 dbg("got EWMH fullscreen state change\n");
-	 switch (e->data.l[1])
+	 switch (e->data.l[0])
 	   {
 	   case _NET_WM_STATE_REMOVE:
+	     if (c->flags & CLIENT_FULLSCREEN_FLAG)
+	       main_client_toggle_fullscreen(c);
 	     break;
 	   case _NET_WM_STATE_ADD:
+	     if (!(c->flags & CLIENT_FULLSCREEN_FLAG))
+	       main_client_toggle_fullscreen(c);
 	     break;
 	   case _NET_WM_STATE_TOGGLE:
 	     main_client_toggle_fullscreen(c);
@@ -187,10 +198,14 @@ ewmh_update(Wm *w)  /* Updates the list of clients on root */
    int  bin_map_cnt = 0;
    SnCycle *current_cycle = w->sn_cycles;
 
+   dbg("%s() called\n", __func__);
+
    while(current_cycle != NULL)
      {
+       dbg("%s() checking cycle %s\n", __func__, current_cycle->bin_name);
        if (current_cycle->xid != None)
 	{
+	  dbg("%s() has xid %li\n", __func__, current_cycle->xid);
 	  bin_map_cnt += (strlen(current_cycle->bin_name) + 32);
 	  dbg("%s(): bin_map_cnt : %i, name: %s\n", __func__, 
 	      bin_map_cnt, current_cycle->bin_name) ;
@@ -311,20 +326,54 @@ ewmh_state_check(Client *c, Atom atom_state_wanted)
 	 {
 	   for(i=0; i < n; i++)
 	     if (value[i] && value[i] == atom_state_wanted)
-	       return True;
-      }
+	       {
+		 if (value) XFree(value);
+		 return True;
+	       }
+	 }
      }
+
+   if (value) XFree(value);
    return False;
 }
 
+void 
+ewmh_set_allowed_actions(Wm *w, Client *c)
+{
+  int num_actions = 1;
+
+  Atom actions[] = {
+    w->atoms[_NET_WM_ACTION_CLOSE],
+    0, 0
+  }; 
+
+  if (c->type == mainwin)
+    actions[num_actions++] = w->atoms[_NET_WM_ACTION_FULLSCREEN];
+
+  if (c->type == dock || c->type == dialog)
+    actions[num_actions++] = w->atoms[_NET_WM_ACTION_MOVE];
+
+  XChangeProperty(w->dpy, c->window, w->atoms[_NET_WM_ALLOWED_ACTIONS],
+		  XA_ATOM, 32, PropModeReplace, (unsigned char *)actions,
+		  num_actions);
+}
 
 void ewmh_set_active(Wm *w)
 {
+  Client *c;
    unsigned long  val[1];
-   if (w->main_client)
-      val[0] = w->main_client->window;
-   else
-      val[0] = None;
+if ((c = wm_get_desktop(w)) != NULL
+	    && (w->flags & DESKTOP_RAISED_FLAG))
+     { 
+       val[0] = c->window; 
+       dbg("%s() setting desktop ( %li ) as active\n", __func__, c->window );
+     }
+   else if (w->main_client)
+     { 
+       val[0] = w->main_client->window; 
+       dbg("%s() setting %li as active\n", __func__, w->main_client->window );
+     }
+   else val[0] = 0;
    
    XChangeProperty(w->dpy, w->root, w->atoms[_NET_ACTIVE_WINDOW] ,
 		   XA_WINDOW, 32, PropModeReplace,
@@ -332,6 +381,67 @@ void ewmh_set_active(Wm *w)
 }
 
 
+#define PING_PENDING_MAX 5
+
+void
+ewmh_hung_app_check(Wm *w)
+{
+  Client *c = NULL;
+
+  if (!w->head_client) return;
+
+  dbg("%s() called\n", __func__ );
+
+  START_CLIENT_LOOP(w, c)
+    {
+      if (c->has_ping_protocol)
+	{
+	  XEvent e;
+
+	  c->pings_pending++;
+
+	  dbg("%s() pinging %s\n", __func__, c->name);
+
+	  e.type = ClientMessage;
+	  e.xclient.window = c->window;
+	  e.xclient.message_type = w->atoms[WM_PROTOCOLS];
+	  e.xclient.format = 32;
+	  e.xclient.data.l[0] = w->atoms[_NET_WM_PING];
+
+	  /* To save a load of code bloat, we just set the timestamp
+	     to the client window ID. This could be slightly evil but
+	     makes things much more compact. 
+	  */
+	  e.xclient.data.l[1] = c->window;
+	  XSendEvent(w->dpy, c->window, False, 0, &e);
+
+	  if (c->pings_pending > PING_PENDING_MAX)
+	    {
+	      /* XXX
+                  - move to own func ? so client_common can use too ?
+                  - check for cmd line param to launch helper app.
+	      */
+	      char buf[257];
+
+	      if (gethostname (buf, sizeof(buf)-1) == 0)
+		{
+		  if (!strcmp (buf, c->host_machine))
+		    {
+		      if (kill (c->pid, 9) < 0)
+			fprintf(stderr, "matchbox: kill -9 on %s failed.\n",
+				c->name);
+		    }
+		}
+	      else fprintf(stderr, "matchbox: gethostname failed\n");
+
+	      dbg("%s() killing app %s as png timeout\n",
+		  __func__, c->name);
+	    }
+	}
+    }
+  END_CLIENT_LOOP(w, c);
+}
+
 static void set_supported(Wm *w) /*  */
 {
   int num_supported = 0;
@@ -356,6 +466,10 @@ static void set_supported(Wm *w) /*  */
     w->atoms[_NET_SHOW_DESKTOP],
     w->atoms[_NET_WM_NAME],
     w->atoms[_NET_WM_ICON],
+    w->atoms[_NET_WM_ALLOWED_ACTIONS],
+    w->atoms[_NET_WM_ACTION_MOVE],
+    w->atoms[_NET_WM_ACTION_FULLSCREEN],
+    w->atoms[_NET_WM_ACTION_CLOSE],
     0, 0
    };
 
@@ -433,7 +547,7 @@ ewmh_get_icon_prop_data(Wm *w, Window win)
   int result;
 
   result =  XGetWindowProperty (w->dpy, win, w->atoms[_NET_WM_ICON],
-				0, 10000L,
+				0, 100000L,
 				False, XA_CARDINAL,
 				&type, &format, &n_items,
 				&bytes_after, (unsigned char **)&data);
