@@ -27,16 +27,22 @@
 
 void ewmh_init(Wm *w);
 
+void ewmh_init_props(Wm *w);
+
 void ewmh_update(Wm *w);
 
 void ewmh_set_active(Wm *w);
 
-int  ewhm_handle_root_message(Wm *w, XClientMessageEvent *e);
+int  ewmh_handle_root_message(Wm *w, XClientMessageEvent *e);
 
 unsigned char *ewmh_get_utf8_prop(Wm *w, Window win, Atom req_atom);
 
 Bool ewmh_state_check(Client *c, Atom atom_state_wanted);
 
+void ewmh_set_allowed_actions(Wm *w, Client *c);
+
+void ewmh_hung_app_check(Wm *w);
+
 #ifndef REDUCE_BLOAT
 
 int *ewmh_get_icon_prop_data(Wm *w, Window win);
