@@ -21,31 +21,38 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
+
+#include "structs.h"
+#include "wm.h"
+#include "misc.h"
+
+
 #ifdef USE_XFT
 #include <X11/Xft/Xft.h>
 #endif
 
-#include "wm.h"
-#include "misc.h"
-#include "structs.h"
 
 int main(int argc, char **argv)
 {
    wm *w;
    struct sigaction act;
-
+   memset(&act, 0, sizeof(struct sigaction));
+   
    act.sa_handler = sig_handler;
    act.sa_flags = 0;
    sigaction(SIGTERM, &act, NULL);
-   sigaction(SIGINT, &act, NULL);
-   sigaction(SIGHUP, &act, NULL);
+   sigaction(SIGINT, &act,  NULL);
+   sigaction(SIGHUP, &act,  NULL);
    sigaction(SIGCHLD, &act, NULL);
 
+   signal (SIGCHLD, SIG_IGN); 	/* as now we exec via keyboard */
+
    /*   
 #ifdef DEBUG
    _Xdebug =1;
 #endif 
    */
+
    w = wm_new(argc, argv);
 
    wm_init_existing(w);
