@@ -14,7 +14,7 @@
 */
 
 /*
-  $Id: main_client.c,v 1.94 2003/04/06 21:24:31 mallum Exp $
+  $Id: main_client.c,v 1.102 2003/07/06 22:47:10 mallum Exp $
 */
 
 #include "main_client.h"
@@ -49,8 +49,14 @@ main_client_new(wm *w, Window win)
 void
 main_client_check_for_state_hints(Client *c)
 {
+  dbg("%s() checking for fullscreen: %s\n", __func__,
+      XGetAtomName(c->wm->dpy, c->wm->atoms[WINDOW_STATE_FULLSCREEN]));
+
   if (ewmh_state_check(c, c->wm->atoms[WINDOW_STATE_FULLSCREEN]))
-    c->flags ^= CLIENT_FULLSCREEN_FLAG;
+    {
+      c->flags ^= CLIENT_FULLSCREEN_FLAG;
+      dbg("%s() client is fullscreen\n", __func__);
+    }
 }
 
 void
@@ -79,20 +85,23 @@ main_client_configure(Client *c)
   int h = wm_get_offsets_size(c->wm, SOUTH, NULL, True);
 
    if ( c->flags & CLIENT_FULLSCREEN_FLAG )
-     c->y = 0;
+     { 
+       dbg("%s() window is fullscreen\n", __func__ );
+       c->y = 0;  
+       c->x = 0;
+       c->height = c->wm->dpy_height;
+       c->width  = c->wm->dpy_width;
+     }
    else
-     c->y = wm_get_offsets_size(c->wm, NORTH, NULL, False) + frm_size;
+     {
+       c->y = wm_get_offsets_size(c->wm, NORTH, NULL, False) + frm_size;
+       c->x = wm_get_offsets_size(c->wm, WEST,  NULL, False) + offset_west;
+       c->width  = c->wm->dpy_width - ( offset_east + offset_west ) 
+	 - wm_get_offsets_size(c->wm, EAST,  NULL, False)
+	 - wm_get_offsets_size(c->wm, WEST,  NULL, False);
 
-   c->x = wm_get_offsets_size(c->wm, WEST,  NULL, False) + offset_west;
-       
-   c->width  = c->wm->dpy_width - ( offset_east + offset_west ) 
-     - wm_get_offsets_size(c->wm, EAST,  NULL, False)
-     - wm_get_offsets_size(c->wm, WEST,  NULL, False);
-
-   c->height = c->wm->dpy_height - c->y - h - offset_south;
-
-   if ( c->flags & CLIENT_FULLSCREEN_FLAG )
-      c->height = c->wm->dpy_height;
+       c->height = c->wm->dpy_height - c->y - h - offset_south;
+     }
 
    dbg("%s() configured as %i*%i+%i+%i, frame size is %i\n", __func__, c->width, c->height, c->x, c->y, frm_size);
    
@@ -205,30 +214,12 @@ void
 main_client_toggle_fullscreen(Client *c)
 {
   XGrabServer(c->wm->dpy);
-  
-  if (c->flags & CLIENT_FULLSCREEN_FLAG )
-    {
-      int title_height = theme_frame_defined_height_get(c->wm->mbtheme, FRAME_MAIN)
-	+ wm_get_offsets_size(c->wm, NORTH, c, False);
-      c->height = c->wm->dpy_height 
-	- title_height
-	- wm_get_offsets_size(c->wm, SOUTH, c, False);
 
-      XMoveResizeWindow(c->wm->dpy, c->frame, c->x, 0, c->width, 
-			c->height + title_height );
-      XResizeWindow(c->wm->dpy, c->window, c->width, c->height);
-    } else {
-      c->height = c->wm->dpy_height;
-      XMoveResizeWindow(c->wm->dpy, c->frame, c->x, 
-			-main_client_title_height(c),
-			c->width, c->height+main_client_title_height(c));
-      XResizeWindow(c->wm->dpy, c->window, c->width, c->height);
-    }
-  
-  XUngrabServer(c->wm->dpy);
-  
   c->flags ^= CLIENT_FULLSCREEN_FLAG;
+  main_client_configure(c);
+  main_client_move_resize(c);
 
+  XUngrabServer(c->wm->dpy);
 }
 
 /* redraws the frame */
@@ -272,7 +263,6 @@ main_client_redraw(client *c, Bool use_cache)
      client_init_backing_mask(c, c->width + offset_east + offset_west, 
 	        		 c->height + offset_south);
 
-
    theme_frame_paint(c->wm->mbtheme, c, FRAME_MAIN, 0, 0, w, h); 
 
    theme_frame_paint(c->wm->mbtheme, c, FRAME_MAIN_WEST, 
@@ -286,6 +276,7 @@ main_client_redraw(client *c, Bool use_cache)
 		     0, c->height + h, 
 		     c->width + offset_east + offset_west, offset_south); 
 
+
    theme_frame_button_paint(c->wm->mbtheme, c, BUTTON_ACTION_CLOSE, 
 			    INACTIVE, FRAME_MAIN, w, h);
 
@@ -336,7 +327,7 @@ main_client_redraw(client *c, Bool use_cache)
       XShapeCombineMask( c->wm->dpy, c->title_frame, ShapeBounding, 0, 0, 
 			 c->backing_mask, ShapeSet);
     }
-   
+
    XSetWindowBackgroundPixmap(c->wm->dpy, c->title_frame, c->backing);
    XClearWindow(c->wm->dpy, c->title_frame);
 
@@ -387,10 +378,10 @@ void main_client_button_press(client *c, XButtonEvent *e)
 	 main_client_toggle_title_bar(c);
 	 break;
       case BUTTON_ACTION_HELP:
-	client_deleiver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_HELP]);
+	client_deliver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_HELP]);
 	 break;
       case BUTTON_ACTION_ACCEPT:
-	client_deleiver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_ACCEPT]);
+	client_deliver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_ACCEPT]);
 	 break;
 
       case -1: 		 /* Cancelled  */
@@ -475,6 +466,7 @@ main_client_show(Client *c)
    if ((desktop = wm_get_desktop(c->wm)) != NULL)
      XLowerWindow(c->wm->dpy, desktop->window);
 
+
    XMapRaised(c->wm->dpy, c->frame);
    XMapSubwindows(c->wm->dpy, c->frame);
 
@@ -488,6 +480,8 @@ main_client_show(Client *c)
 
    /* deal with transients etc */
    base_client_show(c);
+
+
 }
 
 
@@ -497,6 +491,7 @@ main_client_destroy(Client *c)
    Wm *w = c->wm;
 
    dbg("%s called for %s\n", __func__, c->name);
+  
    /* What main clients are left?, need to decide what nav buttons appear */
 
    if (c == w->main_client)
@@ -509,14 +504,21 @@ main_client_destroy(Client *c)
 	 if (c->wm->flags & SINGLE_FLAG)
 	    c->wm->flags ^= SINGLE_FLAG; /* turn off single flag */
 	 base_client_destroy(c);
-	 wm_activate_client(w->head_client);
+
+	 /* Check for a desktop win and update  */
+	 if (!(c->wm->flags & DESKTOP_RAISED_FLAG))
+	   {
+	     wm_toggle_desktop(c->wm);
+	   }
+	 else wm_activate_client(w->head_client);
+
 	 return;
       }
    }
 
    base_client_destroy(c); 
 
-   wm_activate_client(w->head_client);   
+   wm_activate_client(w->main_client);   
 
    if (w->main_client == client_get_prev(w->main_client, mainwin))
      {  /* Is only 1 main_client left  */
