@@ -49,17 +49,22 @@ static struct frame_lookup_t
   char* name;
   int   id;
 } frame_lookup[] = {
-  { "main",        FRAME_MAIN         },
-  { "main-east",   FRAME_MAIN_EAST    },
-  { "main-west",   FRAME_MAIN_WEST    },
-  { "main-south",  FRAME_MAIN_SOUTH   },
-  { "dialog",      FRAME_DIALOG       },
-  { "dialog-east", FRAME_DIALOG_EAST  },
-  { "dialog-west", FRAME_DIALOG_WEST  },
-  { "dialog-south",FRAME_DIALOG_SOUTH },
-  { "utility-min", FRAME_UTILITY_MIN  },
-  { "utility-max", FRAME_UTILITY_MAX  },
-  { "menu",        FRAME_MENU         },
+  { "main",          FRAME_MAIN         },
+  { "main-east",     FRAME_MAIN_EAST    },
+  { "main-west",     FRAME_MAIN_WEST    },
+  { "main-south",    FRAME_MAIN_SOUTH   },
+  { "dialog",        FRAME_DIALOG       },
+  { "dialog-north",   FRAME_DIALOG_NORTH },
+  { "dialog-east",   FRAME_DIALOG_EAST  },
+  { "dialog-west",   FRAME_DIALOG_WEST  },
+  { "dialog-south",  FRAME_DIALOG_SOUTH },
+  { "utility-min",   FRAME_UTILITY_MIN  },
+  { "utility-max",   FRAME_UTILITY_MAX  },
+  { "menu",          FRAME_MENU         },
+  { "message",       FRAME_MSG         },
+  { "message-east",  FRAME_MSG_EAST    },
+  { "message-west",  FRAME_MSG_WEST    },
+  { "message-south", FRAME_MSG_SOUTH   },
 };
 
 #define frame_lookup_cnt (sizeof(frame_lookup)/sizeof(struct frame_lookup_t))
@@ -96,7 +101,7 @@ theme_get_str_length(MBTheme     *theme,
 			&extents);
    else
      XftTextExtents8(theme->wm->dpy, font->font, 
-		     (unsigned char *) str, str_len, &extents);
+		     (unsigned char *)str, str_len, &extents);
    return extents.width;
 #else
    return XTextWidth(font->font, str, str_len);
@@ -168,6 +173,7 @@ void theme_frame_icon_paint(MBTheme *t, Client *c,
 
   mb_pixbuf_img_composite(t->wm->pb, img_dest, img, x, y);
 
+  if (img != NULL) mb_pixbuf_img_free(t->wm->pb, img);
 }
 
 
@@ -276,23 +282,49 @@ theme_frame_button_paint(MBTheme *theme,
 	      
 	      dbg("%s new button is NOT input only\n", __func__);
 
-	      img_backing = mb_pixbuf_img_new(theme->wm->pb, 
-					      button_w, button_h);
-	      
+	      /*
+	      img_backing = mb_pixbuf_img_rgb_new(theme->wm->pb, 
+						  button_w, button_h);
+	      */
 	      pxm_button = XCreatePixmap(theme->wm->dpy, theme->wm->root,
 					 button_w, button_h, 
 					 theme->wm->pb->depth);
 
-	      dbg("%s() copying +%i+%i %ix%i \n", __func__,
+	      dbg("%s() copying +%i+%i %ix%i cache %ix%i\n", __func__,
 		  button_x, button_y,
-		  button_w, button_h );
-	      
+		  button_w, button_h, 
+		  mb_pixbuf_img_get_width(theme->img_caches[frame_type]),
+		  mb_pixbuf_img_get_height(theme->img_caches[frame_type]));
+
+	      /*
 	      mb_pixbuf_img_copy(theme->wm->pb, img_backing,
 				 theme->img_caches[frame_type],
 				 button_x, button_y,
 				 button_w, button_h,
 				 0, 0 );
-	      
+	      */
+
+	      if (c->type == mainwin || c->type == toolbar )
+		{
+		  img_backing = mb_pixbuf_img_rgb_new(theme->wm->pb, 
+						      button_w, button_h);
+		  mb_pixbuf_img_copy(theme->wm->pb, img_backing,
+				     theme->img_caches[frame_type],
+				     button_x, button_y,
+				     button_w, button_h,
+				     0, 0 );
+		}
+	      else
+		{
+		  img_backing 
+		    = mb_pixbuf_img_new_from_x_drawable (theme->wm->pb, 
+							 c->backing, 
+							 None,
+							 button_x, button_y,
+							 button_w, button_h,
+							 False);
+		}
+
 	      if (state == ACTIVE)
 		{
 		  if (button->img_active->width > button_w)
@@ -305,13 +337,13 @@ theme_frame_button_paint(MBTheme *theme,
 		  else
 		    copy_h = button->img_active->height;
 
+		  mb_pixbuf_img_copy_composite_with_alpha(theme->wm->pb, 
+							  img_backing,
+							  button->img_active, 
+							  0, 0, copy_w, copy_h,
+							  0, 0, 
+							  button->img_active_blend);
 
-		  mb_pixbuf_img_copy_composite_blend(theme->wm->pb, 
-						     img_backing,
-						     button->img_active, 
-						     0, 0, copy_w, copy_h,
-						     0, 0, 
-						     button->img_active_blend);
 		} else {
 
 		  if (button->img_inactive->width > button_w)
@@ -325,7 +357,7 @@ theme_frame_button_paint(MBTheme *theme,
 		    copy_h = button->img_inactive->height;
 
 
-		  mb_pixbuf_img_copy_composite_blend(theme->wm->pb, 
+		  mb_pixbuf_img_copy_composite_with_alpha(theme->wm->pb, 
 						     img_backing,
 						     button->img_inactive, 
 						     0, 0, copy_w, copy_h,
@@ -526,7 +558,7 @@ theme_frame_paint( MBTheme *theme,
   struct list_item* layer_list_item;
 
   int title_bytes = 0, title_width = 0, 
-    frame_label_width = 0, frame_label_x = 0;
+    local_frame_label_width = 0, local_frame_label_x = 0;
 
   Bool free_img = False;
 
@@ -540,13 +572,17 @@ theme_frame_paint( MBTheme *theme,
       && theme->img_caches[frame_type] != NULL)
     {
       img = theme->img_caches[frame_type];
-      have_img_cached = TRUE;
+      have_img_cached = True;
     }
   else
     {
       if (theme->img_caches[frame_type] != NULL)
-	mb_pixbuf_img_free(theme->wm->pb, theme->img_caches[frame_type]);
-      img = theme->img_caches[frame_type] = mb_pixbuf_img_new(theme->wm->pb,dw,dh);
+	  mb_pixbuf_img_free(theme->wm->pb, theme->img_caches[frame_type]);
+     if (c->backing_mask != None) /* Need alpha chan for shape */
+       theme->img_caches[frame_type] = mb_pixbuf_img_rgba_new(theme->wm->pb,dw,dh);
+     else
+       theme->img_caches[frame_type] = mb_pixbuf_img_rgb_new(theme->wm->pb,dw,dh);
+      img = theme->img_caches[frame_type];
     }
 
   layer_list_item = frame->layers;
@@ -567,11 +603,11 @@ theme_frame_paint( MBTheme *theme,
 					 c->name, 
 					 title_bytes,  c->name_is_utf8);
 	  
-      frame_label_width = param_get(frame, layer_label->w, dw);
+      local_frame_label_width = param_get(frame, layer_label->w, dw);
 
-      if (title_width > frame_label_width)
+      if (title_width > local_frame_label_width)
 	{
-	  while (title_width > frame_label_width && title_bytes > 0)
+	  while (title_width > local_frame_label_width && title_bytes > 0)
 	    {
 	      title_width = theme_get_str_length(theme, 
 						 layer_label->label->font, 
@@ -582,22 +618,22 @@ theme_frame_paint( MBTheme *theme,
 	    }
 	}
 	
-      frame->label_w = frame_label_width;
+      frame->label_w = local_frame_label_width;
+
+      local_frame_label_x = param_get(frame, layer_label->x, dw);
 
-      frame_label_x = param_get(frame, layer_label->x, dw);
 
       switch(local_justify)
-	{
-	case ALIGN_LEFT:
-	  frame->label_x = frame_label_x;
-	  break;
-	case ALIGN_RIGHT:
-	  frame->label_x = frame_label_x + frame_label_width - title_width;
-	  break;
-	case ALIGN_CENTER:
-	  frame->label_x = frame_label_x + ((frame_label_x - title_width) / 2);
-	  break;
-	}
+        {
+        case ALIGN_LEFT:
+          break;
+        case ALIGN_RIGHT:
+          local_frame_label_x = local_frame_label_x + local_frame_label_width - title_width;
+          break;
+        case ALIGN_CENTER:
+          local_frame_label_x = local_frame_label_x +((local_frame_label_width - title_width)/2);
+          break;
+        }
     }
 
   if (!have_img_cached)
@@ -647,7 +683,7 @@ theme_frame_paint( MBTheme *theme,
 	  XftDrawStringUtf8(c->xftdraw, 
 			    &layer_label->label->col->xftcol, 
 			    layer_label->label->font->font,
-			    frame_label_x, 
+			    local_frame_label_x, 
 			    param_get(frame, layer_label->y, dh)
 			    + layer_label->label->font->font->ascent,
 			    (unsigned char *) c->name, 
@@ -656,7 +692,7 @@ theme_frame_paint( MBTheme *theme,
       else XftDrawString8(c->xftdraw, 
 			  &layer_label->label->col->xftcol, 
 			  layer_label->label->font->font,
-			  frame_label_x,
+			  local_frame_label_x,
 			  param_get(frame, layer_label->y, dh)
 			  + layer_label->label->font->font->ascent,
 			  (unsigned char *) c->name, 
@@ -666,7 +702,7 @@ theme_frame_paint( MBTheme *theme,
       XSetFont(theme->wm->dpy, theme->gc, layer_label->label->font->font->fid);
       XSetForeground(theme->wm->dpy, theme->gc, 
 		     layer_label->label->col->xcol.pixel);
-      XDrawString(theme->wm->dpy, c->backing, theme->gc, frame_label_x, 
+      XDrawString(theme->wm->dpy, c->backing, theme->gc, local_frame_label_x, 
 		  param_get(frame, layer_label->y, dh) 
 		  + layer_label->label->font->font->ascent,
 		  c->name, title_bytes);
@@ -898,6 +934,14 @@ theme_frame_supports_button_type(MBTheme *theme,
     return False;
 }
 
+Bool
+theme_has_frame_type_defined(MBTheme *theme, int frame_type)
+{
+  if (list_find_by_id (theme->frames, frame_type))
+    return True;
+  else
+    return False;
+}
 
 int
 theme_frame_button_get_x_pos(MBTheme *theme, 
@@ -1397,8 +1441,11 @@ mbtheme_frame_free (MBTheme      *theme,
 
       cur = next;
     }
+
   frame->buttons = NULL;
 
+  if (frame->options) free(frame->options);
+
   free(frame);
 }
 
@@ -1621,6 +1668,10 @@ void
 mbtheme_font_free (MBTheme     *theme, 
 		   MBThemeFont *font)
 {
+#ifdef USE_XFT
+  if (font->font) XftFontClose(theme->wm->dpy, font->font);
+#endif
+  
   free(font);
 }
 
@@ -1697,14 +1748,11 @@ mbtheme_free (Wm      *w,
 {
   struct list_item* next = NULL, *cur = NULL;
 
-  XFreeGC(w->dpy, theme->band_gc);
-
-  list_destroy(&theme->frames);
-
   cur = theme->frames;
   while (cur != NULL)
     {
       next = cur->next;
+      if (cur->name) free(cur->name);
       mbtheme_frame_free (theme, (MBThemeFrame*)cur->data);
       free(cur);
       cur = next;
@@ -1715,6 +1763,7 @@ mbtheme_free (Wm      *w,
   while (cur != NULL)
     {
       next = cur->next;
+      if (cur->name) free(cur->name);
       mb_pixbuf_img_free(w->pb, (MBPixbufImage *)cur->data);
       free(cur);
       cur = next;
@@ -1725,6 +1774,7 @@ mbtheme_free (Wm      *w,
   while (cur != NULL)
     {
       next = cur->next;
+      if (cur->name) free(cur->name);
       mbtheme_color_free(theme, (MBThemeColor*)cur->data);
       free(cur);
       cur = next;
@@ -1735,15 +1785,21 @@ mbtheme_free (Wm      *w,
   while (cur != NULL)
     {
       next = cur->next;
+      if (cur->name) free(cur->name);
       mbtheme_font_free(theme, (MBThemeFont*)cur->data);
       free(cur);
       cur = next;
     }
   theme->fonts = NULL;
 
-  /* list_destroy(&theme->pictures); - no pictures as yet */
+  if (theme->gc) XFreeGC(w->dpy, theme->gc);
+  if (theme->band_gc) XFreeGC(w->dpy, theme->band_gc);
+  if (theme->mask_gc) XFreeGC(w->dpy, theme->mask_gc);
+
+  theme_img_cache_clear_all (theme);
 
   free(theme);
+
   w->mbtheme = NULL;
 }
 
@@ -1775,15 +1831,17 @@ mbtheme_switch (Wm   *w,
 
   theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
 
-  START_CLIENT_LOOP(w, p);   
-  {
-    client_buttons_delete_all(p);
-    p->configure(p);
-    p->move_resize(p);
-    p->redraw(p, False);
-  }
-  END_CLIENT_LOOP(w, p);
-
+  if (w->head_client)
+    {
+      START_CLIENT_LOOP(w, p);   
+      {
+	client_buttons_delete_all(p);
+	p->configure(p);
+	p->move_resize(p);
+	p->redraw(p, False);
+      }
+      END_CLIENT_LOOP(w, p);
+    }
 
   /* XXX fix if desktop is shown */
   if (win_active)
@@ -1880,7 +1938,7 @@ mbtheme_init (Wm   *w,
 		     DATADIR, theme_name);
 	  }
       }
-  } 
+  } else { theme_name = DEFAULTTHEMENAME; }
   
   if (!file_exists(theme_filename))
     {
@@ -1902,9 +1960,12 @@ mbtheme_init (Wm   *w,
   
   chdir(theme_path);
 
-  XChangeProperty(w->dpy, w->root, w->mb_theme, XA_STRING, 8,
+  XChangeProperty(w->dpy, w->root, w->atoms[_MB_THEME], XA_STRING, 8,
 		  PropModeReplace, theme_path, strlen(theme_path));
 
+  XChangeProperty(w->dpy, w->root, w->atoms[_MB_THEME_NAME], XA_STRING, 8,
+		  PropModeReplace, theme_name, strlen(theme_name));
+
   free(theme_path);
 
   root_node = xml_parse_file_dom(parser, theme_filename);
