@@ -21,9 +21,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
-#ifdef USE_XFT
-#include <X11/Xft/Xft.h>
-#endif
 
 #include "structs.h"
 #include "client_common.h"
@@ -42,4 +39,4 @@ void toolbar_client_reparent(Client *c);
 void toolbar_client_redraw(Client *c, Bool use_cache);
 int toolbar_win_offset(Client *c);
 
-#endif _TOOLBAR_CLIENT_H_
+#endif
