@@ -57,45 +57,30 @@ wm_new(int argc, char **argv)
    
    XSetErrorHandler(handle_xerror); 
 
-   w->screen = DefaultScreen(w->dpy);
-   w->root   = RootWindow(w->dpy, w->screen);
-
-   w->dpy_width  = DisplayWidth(w->dpy, w->screen);
-   w->dpy_height = DisplayHeight(w->dpy, w->screen);
-
-   w->next_click_is_not_double = True;
-   
-   w->wm_state = XInternAtom(w->dpy, "WM_STATE", False);
-   w->wm_change_state = XInternAtom(w->dpy, "WM_CHANGE_STATE", False);
-   w->wm_protos   = XInternAtom(w->dpy, "WM_PROTOCOLS", False);
-   w->wm_delete   = XInternAtom(w->dpy, "WM_DELETE_WINDOW", False);
-   w->wm_cmapwins = XInternAtom(w->dpy, "WM_COLORMAP_WINDOWS", False);
-   w->mb_theme    = XInternAtom(w->dpy, "_MB_THEME", False);
-   
+   w->screen         = DefaultScreen(w->dpy);
+   w->root           = RootWindow(w->dpy, w->screen);
+   w->dpy_width      = DisplayWidth(w->dpy, w->screen);
+   w->dpy_height     = DisplayHeight(w->dpy, w->screen);
    w->head_client    = NULL; /* A 'general' pointer for circular list */ 
    w->focused_client = NULL; /* Points to the currently focused client */
    w->main_client    = NULL; /* Points to the current 'main' client
 				- eg the 'BIG' window - */
 
-   
+   w->next_click_is_not_double = True;
+
    sattr.event_mask =  SubstructureRedirectMask
                        |SubstructureNotifyMask
                        |StructureNotifyMask
                        |PropertyChangeMask;
+                       /* |KeymapStateMask; */
 
    /* Tell root win we wanna be wm */
    XChangeWindowAttributes(w->dpy, w->root, CWEventMask, &sattr);
 
 #ifdef USE_XSETTINGS
-
-   /* XXX should probably go in wm_load_config */
-   w->xsettings_client = xsettings_client_new(w->dpy,
-					      DefaultScreen(w->dpy),
-					      wm_xsettings_notify_cb,
-					      NULL,
-					      (void *)w
-					      );
-
+   w->xsettings_client = xsettings_client_new (w->dpy, w->screen,
+					       wm_xsettings_notify_cb,
+					       NULL, (void *)w );
 #endif 
 
 #ifndef STANDALONE
@@ -128,9 +113,11 @@ wm_new(int argc, char **argv)
    keys_init(w);
 #endif
 
+   ewmh_init(w);
+
    mbtheme_init(w, w->config->theme);
 
-   ewmh_init(w);
+   ewmh_init_props(w);
 
    wm_set_cursor_visibility(w, !w->config->no_cursor);
 
@@ -147,10 +134,14 @@ wm_new(int argc, char **argv)
    w->sn_busy_cnt     = 0;
    w->sn_cycles       = NULL;
    w->sn_mapping_list = NULL;
-
 #endif
+
+#ifdef USE_MSG_WIN
+   w->msg_win_queue_head = NULL;
+#endif
+
     
-   w->flags ^= STARTUP_FLAG; 	/* flags should now be 0 */
+   w->flags ^= STARTUP_FLAG; 	/* Remove startup flag */
 
    return w;
 }
@@ -159,11 +150,12 @@ void
 wm_usage(char *progname)
 {
    printf("usage: %s %s [options ...]\n", progname, VERSION);
-   printf("\t-display <string> \n");
-   printf("\t-theme <string> \n");
-   printf("\t-use_titlebar   <yes|no>\n");
-   printf("\t-use_cursor     <yes|no>\n");
-   printf("\t-use_lowlight   <yes|#rrggbbaa>\n");
+   printf("\t-display          <string> \n");
+   printf("\t-theme            <string> \n");
+   printf("\t-use_titlebar     <yes|no>\n");
+   printf("\t-use_cursor       <yes|no>\n");
+   printf("\t-use_lowlight     <yes|#rrggbbaa>\n");
+   printf("\t-use_dialog_mode  <free|const|const-horiz>\n");
    printf("\n");
    printf("Compile time options:\n");
 #ifdef DEBUG
@@ -184,6 +176,12 @@ wm_usage(char *progname)
    printf("\tStartup Notification support     no\n");
 #endif
 
+#ifdef USE_MSG_WIN
+   printf("\tMessage Window Support           yes\n");
+#else
+   printf("\tMessage Window Support           no\n");
+#endif
+
 #ifdef USE_EXPAT
    printf("\tExpat support                    yes\n");
 #else
@@ -196,10 +194,16 @@ wm_usage(char *progname)
    printf("\tXSettings support                no\n");
 #endif
 
-#ifndef USE_PNG
-   printf("\tPNG support                    yes\n");
+#ifdef USE_PNG
+   printf("\tPNG support                      yes\n");
 #else
-   printf("\tPNG support                    no\n");
+   printf("\tPNG support                      no\n");
+#endif
+
+#ifdef USE_JPG
+   printf("\tJPG support                      yes\n");
+#else
+   printf("\tJPG support                      no\n");
 #endif
 
 #ifndef STANDALONE
@@ -222,13 +226,14 @@ wm_load_config (wm   *w,
    char *type;
    XrmValue value;
    
-   static int opTableEntries = 5;
+   static int opTableEntries = 6;
    static XrmOptionDescRec opTable[] = {
       {"-theme",       ".theme",           XrmoptionSepArg, (XPointer) NULL},
       {"-use_titlebar",".titlebar",        XrmoptionSepArg, (XPointer) NULL},
       {"-display",     ".display",         XrmoptionSepArg, (XPointer) NULL},
       {"-use_cursor",  ".cursor",          XrmoptionSepArg, (XPointer) NULL},
       {"-use_lowlight",".lowlight",        XrmoptionSepArg, (XPointer) NULL},
+      {"-use_dialog_mode",".dialog",        XrmoptionSepArg, (XPointer) NULL},
    };
    
    XrmInitialize();
@@ -242,13 +247,14 @@ wm_load_config (wm   *w,
    if ( (w->config = malloc(sizeof(Wm_config))) == NULL)
       err("err out of memory");
 
-   /* defaults */
-   w->config->use_title = True;
-   w->config->display_name[0] = '\0';
-   w->config->dbl_click_time = 200;
-   w->config->use_icons = 16;
-   w->config->no_cursor = False;
-   w->config->dialog_shade = False;   
+   /* config defaults */
+   w->config->use_title        = True;
+   w->config->display_name[0]  = '\0';
+   w->config->dbl_click_time   = 200;
+   w->config->use_icons        = 16;
+   w->config->no_cursor        = False;
+   w->config->dialog_shade     = False;   
+   w->config->dialog_stratergy = WM_DIALOGS_STRATERGY_CONSTRAINED;
 
    if (XrmGetResource(rDB, "matchbox.display",
 		      "Matchbox.Display",
@@ -278,7 +284,8 @@ wm_load_config (wm   *w,
 		      &type, &value) == True)
    {
 #ifdef STANDALONE
-     fprintf(stderr, "This matchbox build does not support themeing\n");
+     fprintf(stderr, 
+	     "matchbox: This matchbox build does not support themeing\n");
 #else
      w->config->theme = (char *)malloc(sizeof(char)*(value.size+1));
      strncpy(w->config->theme, value.addr, (int) value.size);
@@ -321,7 +328,7 @@ wm_load_config (wm   *w,
 	   w->config->lowlight_params[2] = 0; 
 	   w->config->lowlight_params[3] = 100; 
      	 }
-       if (strlen(value.addr) == 9 && value.addr[0] == '#')
+       else if (strlen(value.addr) == 9 && value.addr[0] == '#')
 	 {
 	   int succsess, result;
 	   succsess = sscanf (value.addr+1, "%x", &result);
@@ -338,10 +345,26 @@ wm_load_config (wm   *w,
 		   w->config->lowlight_params[1],
 		   w->config->lowlight_params[2],
 		   w->config->lowlight_params[3] );
-
 	     }
 	 }
-     }   
+       else fprintf(stderr, "matchbox: unable to parse lowlight option, ignoring...\n");
+     }
+
+   if (XrmGetResource (rDB, "matchbox.dialog", "Matchbox.Dialog",
+		       &type, &value) == True)
+     {
+       if (strncmp (value.addr, "free", (int) value.size) == 0)
+	 {
+	   w->config->dialog_stratergy = WM_DIALOGS_STRATERGY_FREE;
+	 }
+       else if (strncmp (value.addr, "const-horiz", (int) value.size) == 0)
+	 {
+	   w->config->dialog_stratergy 
+	     = WM_DIALOGS_STRATERGY_CONSTRAINED_HORIZ;
+	 }
+       else if (strncmp (value.addr, "const", (int) value.size) != 0)
+	 wm_usage("matchbox");
+     } 
 }
 
 void
@@ -358,7 +381,7 @@ wm_init_existing(Wm *w)
       if (!attr.override_redirect && attr.map_state == IsViewable)
       {
 	 c = wm_make_new_client(w, wins[i]);
-	 c->ignore_unmap++;
+	 if (c) c->ignore_unmap++;
       }
    }
    XFree(wins);
@@ -382,47 +405,43 @@ wm_find_client(Wm *w, Window win, int mode)
     return NULL;
 }
 
+static Bool
+get_xevent_timed(Display* dpy, XEvent* event_return, struct timeval *tv)
+{
+  XFlush(dpy);
+
+  if (XPending(dpy) == 0) 
+    {
+      int fd = ConnectionNumber(dpy);
+      fd_set readset;
+      FD_ZERO(&readset);
+      FD_SET(fd, &readset);
+      if (select(fd+1, &readset, NULL, NULL, tv) == 0) 
+	return False;
+      else {
+	XNextEvent(dpy, event_return);
+	return True;
+      }
+    } else {
+      XNextEvent(dpy, event_return);
+      return True;
+    }
+}
+
 void
 wm_event_loop(Wm* w)
 {
   XEvent ev;
-
-#ifdef USE_LIBSN
+  int hung_app_timer = 0;
   struct timeval tvt;
-  int xfd;
-  fd_set fd;
-  Bool have_x_event;
-
-  xfd = ConnectionNumber (w->dpy);
-#endif
 
   for (;;) 
     {
-#ifdef USE_LIBSN
-
-    if (w->sn_busy_cnt)
-      {
-	have_x_event = False;
-
-	while (!have_x_event)
-	  {
-	    tvt.tv_usec = 0;
-	    tvt.tv_sec = 1;
+      tvt.tv_usec = 0;
+      tvt.tv_sec = 1;
       
-	    FD_ZERO (&fd);
-	    FD_SET (xfd, &fd);
-
-	    if (!XPending(w->dpy) && select (xfd+1, &fd, NULL, NULL, &tvt) == 0)
-	      {
-		dbg("%s() select timeout, checking running sn's\n", __func__ );
-		wm_sn_timeout_check (w);
-	      }
-	    else have_x_event = True;
-	  }
-      }
-    
-#endif
-	XNextEvent(w->dpy, &ev);
+      if (get_xevent_timed(w->dpy, &ev, &tvt))
+      {
 	switch (ev.type) 
 	  {
 	  case ButtonPress:
@@ -445,16 +464,15 @@ wm_event_loop(Wm* w)
 	    wm_handle_keypress(w, &ev.xkey); break;
 	  case PropertyNotify:
 	    wm_handle_property_change(w, &ev.xproperty); break;
-	    
-	    /* TODO add these
-	       case ColormapNotify:
-	       handle_colormap_change(&ev.xcolormap); break;
-	    */
+#ifndef NO_KBD
+	  case KeymapNotify:
+	    /* keys_reinit(w); XXX why does this always get called ? */
+	    break;
+#endif
 	  default:
 	    dbg("%s() ignoring event->type : %d", __func__, ev.type);
 	    break;
 	  }
-	
 #ifdef USE_XSETTINGS
 	if (w->xsettings_client != NULL)
 	  xsettings_client_process_event(w->xsettings_client, &ev);
@@ -462,10 +480,31 @@ wm_event_loop(Wm* w)
 
 #ifdef USE_LIBSN
 	sn_display_process_event (w->sn_display, &ev);
-	wm_sn_timeout_check (w);
 #endif
-      }
+      } else {
 
+	/* No X event poll checks here */
+#ifdef USE_LIBSN
+	if (w->sn_busy_cnt)
+	  wm_sn_timeout_check (w);
+#endif      
+
+#ifdef USE_MSG_WIN
+	if (w->msg_win_queue_head)
+	  {
+	    /* dbg("%s() checking message queue\n", __func__); */
+	    wm_msg_win_queue_process (w);
+	  }
+#endif
+
+	/* check for hung apps every five seconds */
+	if (++hung_app_timer > 5)
+	  {
+	    hung_app_timer = 0;
+	    ewmh_hung_app_check(w);
+	  }
+      }
+    }
 }
 
 void
@@ -483,8 +522,6 @@ wm_handle_button_event(Wm *w, XButtonEvent *e)
         *  o check for _NET_WM_STATE_MODAL - set modal flag ?
         *  o test test test ...
         */
-
-       /* c->show(c); */
        XSetInputFocus(c->wm->dpy, c->window,
 		      RevertToPointerRoot, CurrentTime);
        c->wm->focused_client = c;
@@ -511,18 +548,24 @@ wm_handle_button_event(Wm *w, XButtonEvent *e)
       }
    }
 
-   /* remove menu if its up */
+   /* remove task menu if its up */
    if (w->flags & MENU_FLAG)
    {
       if (c && c->type == menu ) c->button_press(c,e);
 
-      START_CLIENT_LOOP(w,p);
-      if (p->type == menu)
-      { select_client_destroy(p); }
+      START_CLIENT_LOOP(w,p) 
+	{
+	  if (p->type == menu)
+	    {
+	      select_client_destroy(p); 
+	      break;
+	    }
+	}
       END_CLIENT_LOOP(w,p);
       return;
    }
 
+   /* Pass the event on to the window class */
    if (c) c->button_press(c,e);
    
    /* clear double click flag if set */
@@ -535,10 +578,12 @@ wm_handle_keypress(Wm *w, XKeyEvent *e)
 #ifndef NO_KBD
   MBConfigKbdEntry *entry =  w->config->kb->entrys;
   Client *p;
+
 #ifdef USE_LIBSN
   Bool found = False;
   SnCycle *current_cycle = w->sn_cycles;
 #endif 
+
    dbg("%s() called\n", __func__ );
 
    if(w->flags & MENU_FLAG)
@@ -552,10 +597,10 @@ wm_handle_keypress(Wm *w, XKeyEvent *e)
        if (p->type == menu)
 	 {
 	   select_client_event_loop( p, NULL );
+	   select_client_destroy (p);
 	   return;
 	 }
        }
-
    
    while (entry != NULL)
      {
@@ -625,10 +670,10 @@ wm_handle_keypress(Wm *w, XKeyEvent *e)
 	    case KEY_ACTN_EXEC_SN:
 	      wm_sn_exec(w, entry->sdata, entry->sdata, entry->sdata);
 	      break;
-
 #endif
 	    case KEY_ACTN_NEXT_CLIENT:
-	      if (w->main_client)
+	      if (w->main_client
+		  && client_get_next(w->main_client,mainwin) != w->main_client)
 		{
 		  XGrabServer(w->dpy);
 		  base_client_hide_transients(w->main_client);
@@ -638,7 +683,8 @@ wm_handle_keypress(Wm *w, XKeyEvent *e)
 		}
 	      break;
 	    case KEY_ACTN_PREV_CLIENT:
-	      if (w->main_client)
+	      if (w->main_client
+		  && client_get_prev(w->main_client,mainwin) != w->main_client)
 		{
 		  XGrabServer(w->dpy);
 		  base_client_hide_transients(w->main_client);
@@ -657,7 +703,14 @@ wm_handle_keypress(Wm *w, XKeyEvent *e)
 	    case KEY_ACTN_TASK_MENU_ACTIVATE:
 	      select_client_new(w);
 	      break;
-
+	    case KEY_ACTN_HIDE_TITLEBAR:
+	      if (w->main_client) 
+		main_client_toggle_title_bar(w->main_client);
+	      break;
+	    case KEY_ACTN_FULLSCREEN:
+	      if (w->main_client) 
+		main_client_toggle_fullscreen(w->main_client);
+	      break;
 	    }
 	}
       entry = entry->next_entry;
@@ -673,20 +726,21 @@ wm_handle_configure_notify(Wm *w, XConfigureEvent *e)
    
    if (e->window == w->root) /* screen rotation */
    {
-      dbg("configure notify event called on root");
+      dbg("%s() configure notify event called on root", __func__ );
       if (e->width  != w->dpy_width ||
 	  e->height != w->dpy_height)
       {
-	 height_diff = e->height - w->dpy_height;
-	 width_diff  = e->width  - w->dpy_width;
-	 w->dpy_width = e->width; 
-	 w->dpy_height = e->height;
+	height_diff   = e->height - w->dpy_height;
+	width_diff    = e->width  - w->dpy_width;
+	w->dpy_width  = e->width; 
+	w->dpy_height = e->height;
+	
+	if (!w->head_client) return;
+	
+	XGrabServer(w->dpy);
 
-	 if (!w->head_client) return;
-
-	 XGrabServer(w->dpy);
-
-	 theme_img_cache_clear( w->mbtheme,  FRAME_MAIN );
+	 /* Clear any caches so decorations get redrawn */
+	 theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
 	 
 	 START_CLIENT_LOOP(w,p);
 	 {
@@ -725,26 +779,21 @@ wm_handle_configure_notify(Wm *w, XConfigureEvent *e)
 	     case dialog :
 	       dialog_client_configure(p);
 	       break;
-	     case override :
-	     case menu:
-	     case detached:
-	       break;
 	     case desktop:
 	       p->width += width_diff;
 	       p->height += height_diff;
 	       cdesktop = p;
 	       break;
+	     case menu:
+	       break;
 	     }
 
 	   if (p != cdesktop) 	/* we leave desktop till last */
 	     {
 	       p->move_resize(p);
-	   
 	       /* destroy buttons so they get reposioned */
 	       client_buttons_delete_all(p);
-	   
 	       p->redraw(p, False);
-	       
 	       client_deliver_config(p);
 	     }
 	 }
@@ -771,8 +820,7 @@ wm_handle_configure_notify(Wm *w, XConfigureEvent *e)
 }
 
 void
-wm_handle_configure_request (Wm                     *w, 
-			     XConfigureRequestEvent *e)
+wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 {
    Client *c = wm_find_client(w, e->window, WINDOW);
    XWindowChanges xwc;
@@ -790,13 +838,9 @@ wm_handle_configure_request (Wm                     *w,
        return;
      }
    
-   dbg("%s() for win %s - have x: %i vs %i, y: %i vs %i, w: %i vs %i, h: %i vs %i,\n", 
-       __func__, c->name, 
-       c->height, e->height, c->width, e->width,
-       c->x, e->x, c->y, e->y
-       );
+   dbg("%s() for win %s - have x: %i vs %i, y: %i vs %i, w: %i vs %i, h: %i vs %i,\n", __func__, c->name, c->height, e->height, c->width, e->width, c->x, e->x, c->y, e->y );
    
-   if (c->type == dock)
+   if (c->type == dock) 	/* Docks can move */
      {
        if ( c->height != e->height || c->width != e->width
 	    || c->x != e->x || c->y != e->y )
@@ -820,15 +864,17 @@ wm_handle_configure_request (Wm                     *w,
 	   c->destroy(c);
 	   
 	   c = wm_make_new_client(w, win_tmp);
-	   c->ignore_unmap++;
+	   if (c) c->ignore_unmap++;
 	   
         } 
        return;
      } 
    
-   /* allow titleless dialogs to move themselves XXX needs testing */
+   /* allow decoration free dialogs to move themselves */
    if (c->type == dialog  	
-       && c->flags & CLIENT_TITLE_HIDDEN_FLAG)
+       && ( c->flags & CLIENT_TITLE_HIDDEN_FLAG 
+	    || c->wm->config->dialog_stratergy == WM_DIALOGS_STRATERGY_FREE)
+       )
      {
        xwc.width  = e->width;
        xwc.height = e->height;
@@ -854,7 +900,6 @@ wm_handle_configure_request (Wm                     *w,
      {
        if (e->detail == Above && !(w->flags & DESKTOP_RAISED_FLAG))
 	 {
-	   dbg("%s() event detail is above\n", __func__);
 	   wm_activate_client(c);
 	 }
        
@@ -908,16 +953,21 @@ wm_handle_unmap_event(Wm *w, XUnmapEvent *e)
    }
 }
 
-
 void
 wm_handle_expose_event(Wm *w, XExposeEvent *e)
 {
    Client *c = wm_find_client(w, e->window, FRAME);
 
-   if (c && e->count == 0)
+   if (c /* && e->count == 0 */)
    {
+     XEvent ev;
+
+     /* Compress expose events */
+     while (XCheckTypedWindowEvent(w->dpy, e->window, Expose, &ev));
+
      dbg("%s() for %s\n", __func__, c->name);    
-     c->redraw(c, TRUE); /* redraw title from cache */
+
+     c->redraw(c, True); /* redraw title from cache - prolly a no-op */
    }
 }
 
@@ -926,7 +976,9 @@ wm_handle_destroy_event(Wm *w, XDestroyWindowEvent *e)
 {
     Client *c = wm_find_client(w, e->window, WINDOW);
     if (!c) return; 
+
     dbg("%s for %s\n", __func__, c->name);
+
     wm_remove_client(w, c);
 }
 
@@ -937,8 +989,9 @@ wm_handle_client_message(Wm *w, XClientMessageEvent *e)
 
    dbg("%s() messgae type is %li\n", __func__, e->message_type);
 
+   /* Handle messages from mbcontrol */
    if (e->message_type == w->atoms[MB_COMMAND])
-     {				/* Handle messages from mbcontrol */
+     {				
        dbg("%s() mb command requested\n", __func__ );
        switch (e->data.l[0])
 	 {
@@ -950,9 +1003,9 @@ wm_handle_client_message(Wm *w, XClientMessageEvent *e)
 	     unsigned long extra;
 	     int format;
 	     int status;
-	     char * value;
+	     char * value = NULL;
 	     status = XGetWindowProperty(w->dpy, w->root,
-					 w->mb_theme, 0L, 512L, False,
+					 w->atoms[_MB_THEME], 0L, 512L, False,
 					 AnyPropertyType, &realType,
 					 &format, &n, &extra,
 					 (unsigned char **) &value);
@@ -965,6 +1018,7 @@ wm_handle_client_message(Wm *w, XClientMessageEvent *e)
 		 dbg("%s() switching theme to %s\n", __func__, value);
 		 mbtheme_switch(w, value);
 	       }
+	     if (value) XFree(value);
 	     return;
 	   }
 #endif
@@ -999,8 +1053,9 @@ wm_handle_client_message(Wm *w, XClientMessageEvent *e)
        return;
      }
 	
-   if (e->message_type == w->wm_change_state)
+   if (e->message_type == w->atoms[WM_CHANGE_STATE])
      {
+       dbg("%s() messagae type is change state\n", __func__ );
        if (c && e->format == 32 && e->data.l[0] == IconicState)
 	 {
 	   c->iconize(c);
@@ -1008,8 +1063,7 @@ wm_handle_client_message(Wm *w, XClientMessageEvent *e)
        return;
      }
 
-   ewhm_handle_root_message(w, e);   
-
+   ewmh_handle_root_message(w, e);   
 }
 
 void
@@ -1019,55 +1073,53 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
 
   Client *c = wm_find_client(w, e->window, WINDOW);
 
-   if (!c) return;
-   dbg("%s() on %s, atom is %li\n", __func__, c->name, e->atom );
+  if (!c) return;
+  dbg("%s() on %s, atom is %li\n", __func__, c->name, e->atom );
    
-   if (e->atom == XA_WM_NAME && !c->name_is_utf8)
-     {
-       if (c->name) XFree(c->name);
-       XFetchName(w->dpy, c->window, (char **)&c->name);
-       base_client_process_name(c);
-       dbg("%s() XA_WM_NAME change, name is %s\n", __func__, c->name);
-       update_titlebar = True;
-     }
-   
-   else if (e->atom == w->atoms[_NET_WM_NAME])
-     {
-       if (c->name) XFree(c->name);
-       c->name = ewmh_get_utf8_prop(w, c->window, w->atoms[_NET_WM_NAME]);
-       if (c->name)
-	 c->name_is_utf8 = True;
-       else
-	 {
-	   c->name_is_utf8 = False;
-	   XFetchName(w->dpy, c->window, (char **)&c->name);
-	 }
-       base_client_process_name(c);
-       dbg("%s() NET_WM_NAME change, name is %s\n", __func__, c->name);
-       update_titlebar = True;
-     }
-   else  if (e->atom == w->wm_change_state)
-     {
-       dbg("%s() state change, name is %s\n", __func__, c->name);
-       if(client_get_state(c) == WithdrawnState)
-	 c->destroy(c);
-     }
-
-   if (update_titlebar)  c->redraw(c, False);
-
+  if (e->atom == XA_WM_NAME && !c->name_is_utf8)
+    {
+      if (c->name) XFree(c->name);
+      XFetchName(w->dpy, c->window, (char **)&c->name);
+      base_client_process_name(c);
+      dbg("%s() XA_WM_NAME change, name is %s\n", __func__, c->name);
+      update_titlebar = True;
+    }
+  else if (e->atom == w->atoms[_NET_WM_NAME])
+    {
+      if (c->name) XFree(c->name);
+      c->name = ewmh_get_utf8_prop(w, c->window, w->atoms[_NET_WM_NAME]);
+      if (c->name)
+	c->name_is_utf8 = True;
+      else
+	{
+	  c->name_is_utf8 = False;
+	  XFetchName(w->dpy, c->window, (char **)&c->name);
+	}
+      base_client_process_name(c);
+      dbg("%s() NET_WM_NAME change, name is %s\n", __func__, c->name);
+      update_titlebar = True;
+    }
+  else  if (e->atom == w->atoms[WM_CHANGE_STATE])
+    {
+      dbg("%s() state change, name is %s\n", __func__, c->name);
+      if(client_get_state(c) == WithdrawnState)
+	c->destroy(c);
+    }
+  
+  if (update_titlebar)  c->redraw(c, False);
 }
 
 Client*
 wm_make_new_client(Wm *w, Window win)
 {
    Window trans_win;
-   Atom prop, realType;
+   Atom realType;
    unsigned long n;
    unsigned long extra;
    unsigned long  val[1];
    int format;
    int status;
-   Atom * value = NULL;
+   Atom *value = NULL;
    client *c = NULL, *t = NULL, *old_main_client = NULL;
    XWMHints *wmhints;
    int mwm_flags = 0;
@@ -1078,58 +1130,88 @@ wm_make_new_client(Wm *w, Window win)
 
    dbg("%s() initiated\n", __func__);
 
-
-   /* find out what type of window it wants to be ... */
-   prop = XInternAtom(w->dpy, "_NET_WM_WINDOW_TYPE", True);
-   if (prop != None)
-   {
-      status = XGetWindowProperty(w->dpy, win, prop, 0L, 1000000L,
-				  0, XA_ATOM, &realType, &format,
-				  &n, &extra, (unsigned char **) &value);
-      if (status == Success)
-      {
-	 if (realType == XA_ATOM && format == 32)
+   status = XGetWindowProperty(w->dpy, win, w->atoms[WINDOW_TYPE], 
+			       0L, 1000000L, 0, XA_ATOM, &realType, &format,
+			       &n, &extra, (unsigned char **) &value);
+   if (status == Success)
+     {
+       if (realType == XA_ATOM && format == 32)
 	 {
-	    dbg("%s() got type atom\n", __func__);
-	    if (value[0] == w->atoms[WINDOW_TYPE_DOCK])
-	    {
+	   dbg("%s() got type atom\n", __func__);
+	   if (value[0] == w->atoms[WINDOW_TYPE_DOCK])
+	     {
 	       dbg("%s() got dock atom\n", __func__ );
-	       //c = docked_client_new(w, win);
 	       c = dockbar_client_new(w, win);
-	    }
-	    if (value[0] == w->atoms[WINDOW_TYPE_TOOLBAR])
-	    {
+	     }
+	   else if (value[0] == w->atoms[WINDOW_TYPE_TOOLBAR])
+	     {
 	       dbg("%s() got toolbar atom\n", __func__ );
 	       c = toolbar_client_new(w, win);
-	    }
-	    if (value[0] == w->atoms[WINDOW_TYPE_DESKTOP])
-	    {
+	     }
+	   else if (value[0] == w->atoms[WINDOW_TYPE_DESKTOP])
+	     {
 	       dbg("%s() got desktop atom\n", __func__ );
 	       c = desktop_client_new(w, win);
-	    }
-	    if (value[0] == w->atoms[WINDOW_TYPE_SPLASH])
-	    {
+	     }
+	   else if (value[0] == w->atoms[WINDOW_TYPE_SPLASH])
+	     {
 	       dbg("%s() got splash atom\n", __func__ );
 	       c = dialog_client_new(w, win, NULL);
 	       c->flags ^= CLIENT_TITLE_HIDDEN_FLAG;
-	    }
-	    if (value[0] == w->atoms[WINDOW_TYPE_DIALOG])
-	    {
+	     }
+	   else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG])
+	     {
 	       dbg("%s() got type dialog atom\n", __func__ );
 	       c = dialog_client_new(w, win, NULL);
-	    }
-
+	     }
+	   
+	   else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE])
+	     {
+#ifdef USE_MSG_WIN
+	       dbg("%s() got type message atom\n", __func__ );
+	       if (w->msg_win_queue_head == NULL)
+		 {
+		   dbg("%s() queue empty add win to queue\n", __func__ );
+		   wm_msg_win_queue_add(w, win);
+		 }
+	       
+	       if (win == w->msg_win_queue_head->win)
+		 {
+		   dbg("%s() win is queue head, making client\n", __func__ );
+		   c = dialog_client_new(w, win, NULL);
+		   c->flags ^= CLIENT_IS_MESSAGE_DIALOG; 
+		 }
+	       else
+		 {
+		   dbg("%s() win is not queue head adding to queue\n", __func__ );
+		   wm_msg_win_queue_add(w, win);
+		   
+		   dbg("%s() returning from add\n", __func__);
+		   XUngrabServer(w->dpy);
+		   if (value) XFree(value);
+		   return NULL;
+		 }
+#else
+	       fprintf(stderr, 
+		       "matchbox: This matchbox build lacks support for message windows.\n"
+		       "          To support them rebuild matchbox with --enable-message-wins.\n" 
+		       );
+#endif
+	     }
+	   
 	 } 
-      }
-   }
+     }
+
+   if (value) XFree(value);
 
    if ((mwm_flags = mwm_get_decoration_flags(w, win)))
      { /* for now, treat just like a splash  */
        if (!c) c = dialog_client_new(w, win, NULL);
        c->flags ^= mwm_flags;
+       dbg("%s() got MWM flags: %i\n", __func__, c->flags );
      }
 
-   /* check for transient */
+   /* check for transient - ie detect if its a dialog */
    XGetTransientForHint(w->dpy, win, &trans_win);
    
    if ( trans_win && (trans_win != win))
@@ -1168,7 +1250,7 @@ wm_make_new_client(Wm *w, Window win)
       c = main_client_new(w, win);
    }
 
-#ifndef REDUCE_BLOAT
+#ifndef STANDALONE
    if ((c->icon_rgba_data = ewmh_get_icon_prop_data(w, win)) != NULL)
      {
        dbg("%s() : got ewmh icon data, size is %i x %i\n", __func__, 
@@ -1190,10 +1272,6 @@ wm_make_new_client(Wm *w, Window win)
 	       c->icon_mask = wmhints->icon_mask;
 	    }
 	 } else {
-	    /* set default 
-	    c->icon = w->generic_icon;
-	    c->icon_mask = w->generic_icon_mask;
-	    */
 	   c->icon = None;
 	   c->icon_mask = None;
 	 }
@@ -1207,11 +1285,12 @@ wm_make_new_client(Wm *w, Window win)
 		   XA_CARDINAL, 32, PropModeReplace,
 		   (unsigned char *)val, 1);
    
+   ewmh_set_allowed_actions(w, c);
+
    dbg("%s() calling configure method for new client\n", __func__);
 
    if (w->config->no_cursor && c->type != dock)
      XDefineCursor (w->dpy, c->window, blank_curs);
-
    
    c->configure(c);
 
@@ -1230,6 +1309,7 @@ wm_make_new_client(Wm *w, Window win)
    XGrabButton(c->wm->dpy, Button1, 0, c->window, True, ButtonPressMask,
 	       GrabModeSync, GrabModeSync, None, None);
 
+
    ewmh_update(c->wm);
    ewmh_set_active(c->wm);
 
@@ -1242,6 +1322,7 @@ wm_make_new_client(Wm *w, Window win)
      base_client_hide_transients(old_main_client);
 
    XUngrabServer(w->dpy);
+
    XFlush(w->dpy);
 
    return c;
@@ -1397,10 +1478,7 @@ wm_restack(Wm *w, Client *client_changed, signed int change_amount)
 		     p->move_resize(p);
 		     client_deliver_config(p);
 		   }
-
-	       case override :
 	       case menu:
-	       case detached:
 	       case desktop:
 		 break;
 	       }
@@ -1410,6 +1488,11 @@ wm_restack(Wm *w, Client *client_changed, signed int change_amount)
 
  XSync(w->dpy, False);
  XUngrabServer(w->dpy);
+
+ ewmh_update(w); /* Needed to update desktop workspace 
+		    XXX should split ewmh_update() into seperate funcs
+		        so more efficient. 
+		 */
 }
 
 void 
@@ -1493,22 +1576,25 @@ wm_toggle_desktop(Wm *w)
    client *p;
    unsigned long  val[1];
 
-   dbg("%s() called w->flags are : %i \n", __func__, (w->flags & DESKTOP_RAISED_FLAG));
+   dbg("%s() called desktop flag is : %i \n", __func__, 
+       (w->flags & DESKTOP_RAISED_FLAG));
 
-   if (!wm_get_desktop(w)) {
-     /* XXX should we automatically start a desktop here if its summoned ? */
+   if (!wm_get_desktop(w)) 
      return;
-   }
 
    XGrabServer(w->dpy);
    if (w->flags & DESKTOP_RAISED_FLAG)
    { 				/* Desktop is visible, raise everything */
      dbg("%s() Desktop is raised, hiding it\n", __func__);
-     if (w->main_client && w->main_client->type != desktop)
+     if (w->main_client != NULL && w->main_client->type != desktop)
        {
-	 /* w->main_client->show(w->main_client); */
-	 wm_activate_client(w->main_client);
 	 XMapRaised(w->dpy, w->main_client->frame);
+	 wm_activate_client(w->main_client);
+       }
+     else if (!w->main_client || w->main_client->type == desktop)
+       { 			/* Nothing to raise above */
+	 XUngrabServer(w->dpy); 
+	 return;
        }
 
      if (w->flags & DESKTOP_RAISED_FLAG) /* client may reset the flag - yuk!*/
@@ -1519,11 +1605,10 @@ wm_toggle_desktop(Wm *w)
    } else {
      dbg("%s() Desktop is hidden, showing it\n", __func__);
       START_CLIENT_LOOP(w,p)
-	 if ( p->type == desktop) 
+	{
+	  if ( p->type == desktop) 
 	   {
 	     XMapRaised(w->dpy, p->window); /* HACK - show call desktop_show */
-	     ewmh_update(w);
-	     ewmh_set_active(w);
 	     if (client_want_focus(p))
 	       {
 		 dbg("%s() setting input focus on desktop\n", __func__);
@@ -1532,36 +1617,35 @@ wm_toggle_desktop(Wm *w)
 		 w->focused_client = p;
 	       }
 	   }
+	}
       END_CLIENT_LOOP(w,p);
-      START_CLIENT_LOOP(w,p)
-	 if ( p->type == toolbar) 
-	   { p->hide(p); XMapRaised(w->dpy, p->frame); }
-         if ( p->type == dock) 
-             XMapRaised(w->dpy, p->window);
-	 /* make sure any transient for root dialogs are shown  
-	    XXX - is this really wanted ?
-	 */
-	 if (p->type == dialog && p->trans == NULL && p->mapped)
-	    {
-	       p->show(p);
-	    }
 
+      START_CLIENT_LOOP(w,p)
+	{
+	  if ( p->type == toolbar) 
+	    { /* p->hide(p); */ XMapRaised(w->dpy, p->frame); }
+	  if ( p->type == dock) 
+	    XMapRaised(w->dpy, p->window);
+	  if (p->type == dialog && p->trans == NULL && p->mapped)
+	    {
+	      p->show(p);
+	    }
+	}
       END_CLIENT_LOOP(w,p);
+
       if (!(w->flags & DESKTOP_RAISED_FLAG))
 	w->flags ^= DESKTOP_RAISED_FLAG;	
-      
 
       val[0] = 1;
    }
 
-   /* TODO: 
-      move below into seperate ehwm.c func so main_clients can also set 
-      if they update the flag. 
-   */
    XChangeProperty(w->dpy, w->root, w->atoms[_NET_SHOW_DESKTOP],
 		   XA_CARDINAL, 32, PropModeReplace, 
 		   (unsigned char *)val, 1);
 
+   ewmh_update(w);
+   ewmh_set_active(w);
+
    XUngrabServer(w->dpy);
 }
 
@@ -1569,7 +1653,6 @@ void
 wm_set_cursor_visibility(Wm *w, Bool visible)
 {
   /* XXX do we need to free the cursors ? */
-
   if (visible)
     {
       w->config->no_cursor = False;
@@ -1592,6 +1675,9 @@ Client *
 wm_get_desktop(Wm *w)
 {
   Client *p;
+ 
+  if (!w->head_client) return NULL;
+
   START_CLIENT_LOOP(w,p)
     if ( p->type == desktop) return p;
   END_CLIENT_LOOP(w,p);
@@ -1743,6 +1829,8 @@ wm_sn_cycle_update_root_prop(Wm *w)
   char *prop_str = NULL;
   int prop_str_len = 0;
 
+  ewmh_update(w);
+
   if (current_cycle == NULL)
     {
       XDeleteProperty(w->dpy, w->root, w->atoms[MB_CLIENT_STARTUP_LIST]);
@@ -1993,6 +2081,21 @@ wm_lowlight(Wm *w, Client *c)
   Pixmap pxm_tmp;
   XSetWindowAttributes attr;
 
+#ifdef USE_MSG_WIN
+  Client *msg_client = NULL;
+
+  if (w->msg_win_queue_head)
+    {
+      if ((msg_client = wm_find_client(w, w->msg_win_queue_head->win, 
+				       WINDOW)) != NULL)
+	{
+	  msg_client->ignore_unmap++;
+	  XUnmapWindow(w->dpy, msg_client->frame);
+	  XSync(w->dpy, True);
+	}
+    }
+#endif
+
   attr.override_redirect = True;
   attr.event_mask = ChildMask|ButtonPressMask|ExposureMask;
        
@@ -2009,21 +2112,23 @@ wm_lowlight(Wm *w, Client *c)
 			  w->dpy_height ,
 			  w->pb->depth);
   
-  img = mb_pixbuf_img_new_from_drawable(c->wm->pb, w->root, 
-					None, 0, 0,
-					w->dpy_width, 
-					w->dpy_height);
+  img = mb_pixbuf_img_new_from_x_drawable(c->wm->pb, w->root, 
+					  None, 0, 0,
+					  w->dpy_width, 
+					  w->dpy_height,
+					  True);
 
   XMapWindow(w->dpy, c->frame);  
 
   for (x = 0; x < w->dpy_width; x++)
     for (y = 0; y < w->dpy_height; y++)
-      mb_pixbuf_img_composite_pixel(img, x, y, 
-				    w->config->lowlight_params[0],
-				    w->config->lowlight_params[1],
-				    w->config->lowlight_params[2],
-				    w->config->lowlight_params[3] 
-				    );
+      mb_pixbuf_img_plot_pixel_with_alpha(c->wm->pb,
+					  img, x, y, 
+					  w->config->lowlight_params[0],
+					  w->config->lowlight_params[1],
+					  w->config->lowlight_params[2],
+					  w->config->lowlight_params[3] 
+					  );
 
   /* Striped pattern diabled. 
     if ( (y % 6) > 2 )
@@ -2042,3 +2147,107 @@ wm_lowlight(Wm *w, Client *c)
 
 #endif
 }
+
+#ifdef USE_MSG_WIN
+
+void
+wm_msg_win_queue_add(Wm *w, Window win)
+{
+  MsgWinQueue *tmp = w->msg_win_queue_head;
+  int timeout;
+
+  Atom type;
+  int format;
+  long bytes_after;
+  int *data = NULL;
+  long n_items;
+  int result;
+
+  result =  XGetWindowProperty (w->dpy, win, 
+				w->atoms[WINDOW_TYPE_MESSAGE_TIMEOUT],
+				0, 1L,
+				False, XA_CARDINAL,
+				&type, &format, &n_items,
+				&bytes_after, (unsigned char **)&data);
+
+  if (result != Success || data == NULL)
+    {
+      timeout = -1; 		/* No timeout */
+    }
+  else timeout = data[0];
+    
+  if (data) XFree (data);
+
+  if (w->msg_win_queue_head == NULL)
+    {
+      dbg("%s() message queue is empty, adding new head\n", __func__);
+      w->msg_win_queue_head = malloc(sizeof(MsgWinQueue));
+      memset(w->msg_win_queue_head, 0, sizeof(MsgWinQueue));
+
+      w->msg_win_queue_head->win = win;
+      w->msg_win_queue_head->timeout = timeout;
+      return;
+    }
+  
+  dbg("%s() queue has items, adding to back \n", __func__);
+
+  while (tmp->next != NULL) tmp = tmp->next;
+
+  tmp->next = malloc(sizeof(MsgWinQueue));
+  memset(tmp->next, 0, sizeof(MsgWinQueue));
+  tmp->next->win = win;
+  tmp->next->timeout = timeout;
+}
+
+void
+wm_msg_win_queue_pop(Wm *w)
+{
+  MsgWinQueue *tmp;
+
+  dbg("%s() called\n", __func__);
+
+  if (w->msg_win_queue_head == NULL) return;
+
+  tmp = w->msg_win_queue_head;
+
+  w->msg_win_queue_head = w->msg_win_queue_head->next;
+
+  free(tmp);
+
+  if (w->msg_win_queue_head)
+    {
+      XWindowAttributes doh_attr;
+      if (XGetWindowAttributes(w->dpy, w->msg_win_queue_head->win, &doh_attr))
+	{
+	  wm_make_new_client(w, w->msg_win_queue_head->win);
+	}
+      else /* Eek window no longer exists  */
+	{
+	  wm_msg_win_queue_pop(w); /* so pop again */
+	}
+    }
+}
+
+void
+wm_msg_win_queue_process(Wm *w)
+{
+  Client      *client_msg = NULL;
+  if (w->msg_win_queue_head == NULL) return;
+
+  /*
+  dbg("%s() queue head has timeout %i\n", 
+      __func__, w->msg_win_queue_head->timeout );
+  */
+
+  if (w->msg_win_queue_head->timeout > 0) w->msg_win_queue_head->timeout--;
+
+  if (w->msg_win_queue_head->timeout == 0) 
+    if ((client_msg = wm_find_client(w, w->msg_win_queue_head->win, 
+				     WINDOW)) != NULL)
+      {
+	wm_remove_client(w, client_msg);
+      }
+}
+
+#endif
+
