@@ -115,4 +115,17 @@ void wm_sn_cycle_remove(Wm *w, Window xid);
 
 #endif
 
+#ifdef USE_MSG_WIN
+
+void
+wm_msg_win_queue_add(Wm *w, Window win);
+
+void
+wm_msg_win_queue_pop(Wm *w);
+
+void
+wm_msg_win_queue_process(Wm *w);
+
+#endif
+
 #endif
